//init params for full_setup, ntrip server, data streaming on ip and transmitter
function getPrefixName()
{
	return YAHOO.util.Dom.get('BaseSetup-PagePrefix').value;
}

function processBaseSetup( config )
{
	//Prepare Antenna Information
	loadAntennaInfo(config.Antenna.List);
	
	var prefixName = getPrefixName();
	//Store active differential messages
	for ( i in config.DifferentialMessages )
	{
		setFieldValue( prefixName + '-DiffMsg-' + config.DifferentialMessages[ i ].Type, config.DifferentialMessages[ i ].Messages );
	}

	// -- Status --
	setFieldValue( prefixName + '-Field-Dynamic', 1 );
	
	//this field actvie only for full_setup.html
	//setFieldChecked( prefixName + '-Field-IsPositionMoving', config.IsPositionMoving );

	setFieldValue( prefixName + '-Field-StationId', config.StationId );
	
	// -- Latitude --
	setFieldValueAsLatitude( prefixName + '-Field-Latitude', config.Latitude);
		
	// -- Longitude --
	setFieldValueAsLongitude( prefixName + '-Field-Longitude', config.Longitude);

	// -- Ellipsoid Height --
	setFieldValueAsDistance( prefixName + '-Field-EllipsoidHeight', config.EllipsoidHeight);

	//In case we have zero positions, we need to show current position
	if ( config.Latitude == 0 && config.Longitude == 0 && config.EllipsoidHeight == 0 )
		showCurrentPos();
				
	// -- Antenna List --
	if (config.Antenna.List != null)
	{
		var names = new Array();
		
		for (i in config.Antenna.List)
		{	
			if (config.Antenna.List[i].AntennaName != null)
			    names.push(config.Antenna.List[i].AntennaName);
		}
			
		setFieldOptions( prefixName + '-Field-OwnAntennaName', names, names);
		addFieldOptions( prefixName + '-Field-OutAntennaName', names, names); // (Keep "OFF" option and add others)
	}

	// -- Own Antenna Name --
	setFieldValue( prefixName + '-Field-OwnAntennaName', config.Antenna.OwnAntennaName);

	// -- Out Antenna Name --
	setFieldValue( prefixName + '-Field-OutAntennaName', config.Antenna.OutAntennaName);

	// -- Antenna Height --
	setFieldValueAsDistance( prefixName + '-Field-AntennaHeight', config.Antenna.AntennaHeight);
	
	// -- Antenna Height Measurement Type --
	setFieldValue( prefixName + '-Field-MeasurementType', config.Antenna.MeasurementType);
	enableSHMP(config.Antenna.MeasurementType);

	// -- Antenna Radius --
	setFieldValueAsDistance( prefixName + '-Field-AntennaRadius', config.Antenna.AntennaRadius);

	// -- Antenna Vertical Offset --
	setFieldValueAsDistance( prefixName + '-Field-AntennaSHMP', config.Antenna.AntennaVerticalOffset);
			
	// -- Antenna Reduction Mode --
	setFieldValue( prefixName + '-Field-AntennaReductionMode', config.Antenna.AntennaReductionMode);

	// -- Reference Elevation Mask --
	setFieldValue( prefixName + '-Field-ElevationMask', config.ReferenceElevationMask);
			
	//Options
	optionsGlobal = config.Options;

	// -- GPS --
	setFieldChecked( prefixName + '-Field-GPS', config.SatelliteGPS);
	
	// -- GLONASS --
	//setFieldChecked( prefixName + '-Field-GLONASS', config.SatelliteGlonass);
	if (optionsGlobal.indexOf("S") != -1)
	{
	    setFieldVisible( prefixName + '-Field-GLONASS', 1);
	    setFieldChecked( prefixName + '-Field-GLONASS', config.SatelliteGlonass);
	} else {
	    setFieldVisible( prefixName + '-Field-GLONASS', 0);
	}
	
	// -- SBAS --
	setFieldChecked( prefixName + '-Field-SBAS', config.SatelliteSbas);

	// -- GALILEO --
	//setFieldChecked( prefixName + '-Field-GALILEO', config.SatelliteGALILEO);
	if (optionsGlobal.indexOf("O") != -1)
	{
	    setFieldVisible(prefixName + '-Field-GALILEO', 1);
	    setFieldChecked( prefixName + '-Field-GALILEO', config.SatelliteGALILEO);
	} else {
	    setFieldVisible(prefixName + '-Field-GALILEO', 0);
	}

	// -- QZSS --
	setFieldChecked( prefixName + '-Field-QZSS', config.SatelliteQZSS);
	
	YAHOO.util.Event.addListener(prefixName + '-Field-GPS', 'click', onFieldGPSChange );
	YAHOO.util.Event.addListener(prefixName + '-Field-GLONASS', 'click', onFieldGLONASSChange );
}

function fillMountPointTable( id, source )
{
	var prefixName = getPrefixName();
	
	// Columns definition
	var myColumnDefs = 
	[
	 	{label:getResource(prefixName + '-MountPoint-Resource-Name'), key:"Name", resizeable:true, sortable:false},
	 	{label:getResource(prefixName + '-MountPoint-Resource-Identifier'), key:"Identifier", resizeable:true, sortable:false}
	];
	
	// Data source
	var myDataSource = new YAHOO.util.DataSource(source);
	myDataSource.responseType = YAHOO.util.DataSource.TYPE_JSARRAY;
	myDataSource.responseSchema = { fields: ["Name","Identifier"] };
	
	// Table configs
	var myConfigs =
	{
		width:"auto",
		height:"auto"
	};
	
	// Localized messages
	myConfigs.MSG_EMPTY = getResource(prefixName + '-MountPoint-Resource-Empty');
	myConfigs.MSG_LOADING = getResource(prefixName + '-MountPoint-Resource-Loading');

	// Create table	
	var table = new YAHOO.widget.DataTable( id, myColumnDefs, myDataSource, myConfigs);
}

function processNtripParameters( config )
{
	var prefixName = getPrefixName();
	
	// -- Network 1 / Ntrip --
	setFieldValue( prefixName + '-Field-Network1-Ntrip-AddressIP', config.Network1.Ntrip.AddressIp);
	setFieldValue( prefixName + '-Field-Network1-Ntrip-AddressPort', config.Network1.Ntrip.AddressPort);
	setFieldValue( prefixName + '-Field-Network1-Ntrip-Mount', config.Network1.Ntrip.Mount);
	//setFieldValue( prefixName + '-Field-Network1-Ntrip-Login', config.Network.Ntrip.Login);
	setFieldValue( prefixName + '-Field-Network1-Ntrip-Password', config.Network1.Ntrip.Password);
	
	// -- Network 2 / Ntrip --
	setFieldValue( prefixName + '-Field-Network2-Ntrip-AddressIP', config.Network2.Ntrip.AddressIp);
	setFieldValue( prefixName + '-Field-Network2-Ntrip-AddressPort', config.Network2.Ntrip.AddressPort);
	setFieldValue( prefixName + '-Field-Network2-Ntrip-Mount', config.Network2.Ntrip.Mount);
	setFieldValue( prefixName + '-Field-Network2-Ntrip-Password', config.Network2.Ntrip.Password);
}

function processDataStreamingParameters( config )
{
	var prefixName = getPrefixName();
	var port = new Array("I1","I2","I3","I4","I5","I6","I7","I8","I9");
	var name = "";
	for (var i in config.IpDataStreaming)
	{
	    name = prefixName + "-Field-" + port[i] + "-";
        setFieldChecked( name + "State", config.IpDataStreaming[i].State);
        setFieldValue( name + "Mode", config.IpDataStreaming[i].Mode);
		
		if ( isEthernetStreamingServerMode(config.IpDataStreaming[i].Mode))  {
			disableField( name + 'IpAddress' );
		}
		
        setFieldValue( name + "Protocol", config.IpDataStreaming[i].Protocol);
        setFieldValue( name + "IpAddress", config.IpDataStreaming[i].IpAddress);
        setFieldValue( name + "IpPort", config.IpDataStreaming[i].IpPort);
        setFieldValue( name + "MessageType", config.IpDataStreaming[i].MessageType);
		showFieldDiffMessages( name + "Messages", config.IpDataStreaming[i].MessageType);
		YAHOO.util.Event.addListener(name + 'Tooltip', 'mouseover', onTooltipOver);
	}
}

function processStream1DiffMessage( config )
{
	var prefixName = getPrefixName();
	
	setFieldValue( prefixName + '-Field-Stream1-MessageType', config.DifferentialStream1.MessageType);
	showDiffMessages( prefixName + '-Label-Stream1-Messages', config.DifferentialStream1.MessageType);
}

function processStream2DiffMessage( config )
{
	var prefixName = getPrefixName();
	
	setFieldValue( prefixName + '-Field-Stream2-MessageType', config.DifferentialStream2.MessageType);
	showDiffMessages( prefixName + '-Label-Stream2-Messages', config.DifferentialStream2.MessageType);
}

function setBaseSetupValidators()
{
	var prefixName = getPrefixName();
	//Units changed
	YAHOO.util.Event.addListener('distances', 'change', onDistanceUnitChanged);
	YAHOO.util.Event.addListener('angles', 'change', onAngleUnitChanged);

	// Status changed
	//YAHOO.util.Event.addListener('FullSetup-Field-Status', 'change', onStatusChanged);
	YAHOO.util.Event.addListener( prefixName + '-ShowCurrentPos', 'click', onShowCurrentPos );
	
	// Measurement Type changed
	YAHOO.util.Event.addListener( prefixName + '-Field-MeasurementType', 'change', onMeasurementTypeChanged);
	
	makeFieldValidator( prefixName + '-Field-Latitude', 'validating_' + 'Field_Latitude');
	makeFieldValidator( prefixName + '-Field-Longitude', 'validating_' + 'Field_Longitude');
	makeFieldValidator( prefixName + '-Field-EllipsoidHeight', 'validating_' + 'Field_EllipsoidHeight');
	makeFieldValidator( prefixName + '-Field-AntennaHeight', 'validating_' + 'Field_AntennaHeight');
	makeFieldValidator( prefixName + '-Field-AntennaRadius', 'validating_' + 'Field_AntennaRadius');
	makeFieldValidator( prefixName + '-Field-AntennaSHMP', 'validating_' + 'Field_AntennaSHMP');
	makeFieldValidator( prefixName + '-Field-ElevationMask', 'validating_' + 'Field_ElevationMask');
	makeFieldValidator( prefixName + '-Field-StationId', 'validating_' + 'Field_StationId' );
}

function setNtripValidators( config )
{
	var prefixName = getPrefixName();
	//makeFieldValidator( prefixName + '-Field-Network1-Ntrip-AddressIP', 'validating_Field_Ip');	
	makeFieldValidator( prefixName + '-Field-Network1-Ntrip-AddressPort', 'validating_Field_Port');
	makeFieldValidator( prefixName + '-Field-Network1-Ntrip-Login', 'validating_Field_Auth');
	makeFieldValidator( prefixName + '-Field-Network1-Ntrip-Password', 'validating_Field_Auth' );
	makeFieldValidator( prefixName + '-Field-Network1-Ntrip-Mount', 	'validating_Field_Ntrip_Mount' );
	
	//makeFieldValidator( prefixName + '-Field-Network2-Ntrip-AddressIP', 'validating_Field_Ip');	
	makeFieldValidator( prefixName + '-Field-Network2-Ntrip-AddressPort', 'validating_Field_Port');
	makeFieldValidator( prefixName + '-Field-Network2-Ntrip-Login', 'validating_Field_Auth');
	makeFieldValidator( prefixName + '-Field-Network2-Ntrip-Password', 'validating_Field_Auth' );
	makeFieldValidator( prefixName + '-Field-Network2-Ntrip-Mount', 	'validating_Field_Ntrip_Mount' );
}

function setDataStreamingValidators()
{
	var prefixName = getPrefixName();
	var port = new Array("I1","I2","I3","I4","I5","I6","I7","I8","I9");
	for ( i in port )
	{
		var fullName = prefixName + '-Field-' + port[ i ] + '-';
		makeFieldValidator( fullName + 'IpAddress', 'validating_Field_Ip' );
		makeFieldValidator( fullName + 'IpPort', 'validating_Field_Port' );
		YAHOO.util.Event.addListener( fullName + 'Mode', 'change', onEthernetStreamingModeChanged );
		YAHOO.util.Event.addListener( fullName + 'MessageType', 'change', onFieldDiffMessageChanged );		
	}
}
function onShowCurrentPos( ev )
{
	showCurrentPos();
}

function showCurrentPos()
{
	var pos = getCurrentPosition();
	
	var prefixName = getPrefixName();
	
	setFieldValueAsLatitude( prefixName + '-Field-Latitude', pos[ 0 ] );
	setFieldValueAsLongitude( prefixName + '-Field-Longitude', pos[ 1 ] );
	setFieldValueAsDistance( prefixName + '-Field-EllipsoidHeight',  pos[ 2 ] );
}

function onDiffMessageChanged( ev )
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	var id = field.id.replace( "Field", "Label" );
	id = id.replace("MessageType", "Messages");
	showDiffMessages( id, field.value );
}

function onFieldDiffMessageChanged( ev )
{
	var field = YAHOO.util.Event.getTarget(ev);
	var id = field.id.replace("MessageType", "Messages");
	showFieldDiffMessages( id, field.value );
}

function showDiffMessages( fieldname, value )
{
	var prefixName = getPrefixName();
	var str = "";
	try
	{
		str = getFieldValue( prefixName + "-DiffMsg-" + value );
	}
	catch( x )
	{
	}
	setItemText( fieldname, str );
}

function showFieldDiffMessages( fieldname, value)
{
	var prefixName = getPrefixName(); 
	var str = getFieldValue( prefixName + "-DiffMsg-" + value );
	setFieldValue( fieldname, str );
}

function onDistanceUnitChanged()
{
	var prefix = getPrefixName();
	updateFieldValueAsDistance( prefix + '-Field-EllipsoidHeight');
	updateFieldValueAsDistance( prefix + '-Field-AntennaHeight');
	updateFieldValueAsDistance( prefix + '-Field-AntennaRadius');
	updateFieldValueAsDistance( prefix + '-Field-AntennaSHMP');
}

function onAngleUnitChanged()
{
	var prefix = getPrefixName();
	updateFieldValueAsLatitude(prefix + '-Field-Latitude');
	updateFieldValueAsLongitude(prefix + '-Field-Longitude');
}

function onStatusChanged(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	enablePosition(field.value);
}

function onMeasurementTypeChanged(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	enableSHMP(field.value);
}

//Ethernet streaming mode staff. convert names and so on
function onEthernetStreamingModeChanged( ev )
{
	var field = YAHOO.util.Event.getTarget( ev );
	//And here convert name id to ip address id
	var pos = field.id.lastIndexOf( '-' );
	var FullName = field.id.substring( 0, pos + 1 );
	if ( isEthernetStreamingServerMode(field.value )) {
		disableField( FullName + 'IpAddress' );
	} else {
	   enableField( FullName + 'IpAddress', true );
	}
}

function isEthernetStreamingServerMode( mode )
{
  return mode == '1';
}

function enablePosition(status)
{
	var prefix = getPrefixName();
	setFieldVisible( prefix + '-Group-Position', status == 0 /* Static Position */);	
}

function enableSHMP(MeasurementType)
{
	var prefix = getPrefixName();
	setFieldVisible( prefix + '-Group-SHMP', MeasurementType == 0 /* Slant Height */);	
}

function validating_Field_StationId( ev, provider )
{
	var field = YAHOO.util.Event.getTarget(ev);
	var valid = isValidInteger( field.value ) && field.value >= 0 && field.value <=4095;
	
	notifyErrorProvider( provider, !valid );
}
function validating_Field_Latitude(ev, provider)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	var value = scanTextAsLatitude(field.value);
	var valid = !isNaN(value) && value >= -90 && value <= 90;
	
	notifyErrorProvider(provider, !valid);
	if (valid)
	{
		setIUFieldValue(field.id, value);
	}
}

function validating_Field_Longitude(ev, provider)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	var value = scanTextAsLongitude(field.value);
	var valid = !isNaN(value) && value >= -180 && value <= 180;
	
	notifyErrorProvider(provider, !valid);
	if (valid)
	{
		setIUFieldValue(field.id, value);
	}
}

function validating_Field_EllipsoidHeight(ev, provider)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	var value = scanTextAsDistance(field.value);
	var valid = !isNaN(value);
	
	notifyErrorProvider(provider, !valid);
	if (valid)
	{
		setIUFieldValue(field.id, value);
	}
}

function validating_Field_AntennaHeight(ev, provider)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	var value = scanTextAsDistance(field.value);
	var valid = !isNaN(value) && value >= 0;
	
	notifyErrorProvider(provider, !valid);
	if (valid)
	{
		setIUFieldValue(field.id, value);
	}
}

function validating_Field_AntennaRadius(ev, provider)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	var value = scanTextAsDistance(field.value);
	var valid = !isNaN(value) && value >= 0 && value <= 6.553;
	
	notifyErrorProvider(provider, !valid);
	if (valid)
	{
		setIUFieldValue(field.id, value);
	}
}

function validating_Field_AntennaSHMP(ev, provider)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	var value = scanTextAsDistance(field.value);
	var valid = !isNaN(value) && value >= 0 && value <= 6.553;
	
	notifyErrorProvider(provider, !valid);
	if (valid)
	{
		setIUFieldValue(field.id, value);
	}
}

function validating_Field_ElevationMask(ev, provider)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	var valid = !isNaN(field.value) && isValidInteger(field.value) && field.value >= 0 && field.value <= 90;
	
	notifyErrorProvider(provider, !valid);
}

function validating_Field_Ip( ev, provider )
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	var valid = (field.value.length <= 32) && (isValidAddressIP(field.value) || isValidHostName(field.value));
	
	notifyErrorProvider(provider, !valid);
}
	
function validating_Field_Port( ev, provider )
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	var valid = !isNaN(field.value) && isValidInteger(field.value) && field.value >= 0 && field.value <= 65535;
	
	notifyErrorProvider(provider, !valid);
}

function validating_Field_Auth( ev, provider )
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	var valid = field.value.length <= 32;
	
	notifyErrorProvider(provider, !valid);
}

function validating_Field_Ntrip_Mount( ev, provider )
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	var valid = field.value.length <= 100;
	
	notifyErrorProvider(provider, !valid);
}

function onFieldGPSChange( ev )
{
	var field = YAHOO.util.Event.getTarget(ev);
	var field_GLO = YAHOO.util.Dom.get(getPrefixName() + '-Field-GLONASS');
	
	if ((isFieldChecked(field) == 0) && (isFieldChecked(field_GLO) == 0)) 
	{
	    setFieldChecked(getPrefixName() + '-Field-GPS', 1);
	}
}

function onFieldGLONASSChange( ev )
{
	var field = YAHOO.util.Event.getTarget(ev);
	var field_GPS = YAHOO.util.Dom.get(getPrefixName() + '-Field-GPS');
	
	if (!isFieldChecked(field) && !isFieldChecked(field_GPS)) 
	{
	    setFieldChecked(getPrefixName() + '-Field-GLONASS', 1);
	}
}
