function initBluetoothModem()
{
	// Define the callbacks for the asyncRequest
	var callbacks = {
		
		success : function (o) {

			YAHOO.log("RAW JSON DATA: " + o.responseText);

			// Process the JSON data returned from the server
			var data;
			try {
				data = YAHOO.lang.JSON.parse(o.responseText);
			}
			catch (x) {
				YAHOO.log("ERROR: JSON Parse failed!");

				// Retry
				setTimeout("initBluetoothModem()", 5000);

				return;
			}

			YAHOO.log("PARSED DATA: " + YAHOO.lang.dump(data));

			// Get config
			var config = data.ConfigBlueToothModem;
			if (config == null) return;

			// Get BT config
			var bt = config.BlueTooth;
			if (bt == null) return;

			// -- BT Address --
			setFieldValue('BTModem-Field-BT-Address', bt.Address);
			
			// -- BT Device Name -- 
			setFieldValue('BTModem-Field-BT-Name', bt.Name);
			
			// -- BT Pin Code --
			setFieldValue('BTModem-Field-BT-Pin', bt.Pin);
			setFieldChecked('BTModem-Field-BT-Secured', bt.Pin != "-1");
			enableBtPin(bt.Pin != "-1", bt.Pin);
			
			// Get Modem config
			var modem = config.Modem;
			if (modem == null) return;

			// -- Modem Power --
			checkRadioButton(
				new Array(
					'BTModem-Field-Modem-Power-On', 
					'BTModem-Field-Modem-Power-Off'), modem.Power);
					
			// -- Modem Power Mode --
			checkRadioButton(
				new Array(
					'BTModem-Field-Modem-PowerMode-Auto', 
					'BTModem-Field-Modem-PowerMode-Manual'), modem.PowerMode);

			// -- Modem Band --
			setFieldValue('BTModem-Field-Modem-Band', modem.Band);
			
            // -- Modem 2G Only --
			setFieldChecked('BTModem-Field-Modem-2GOnly', modem.TwoGOnly == 1);
			setFieldVisible('BTModem-Field-Modem-2GOnly', modem.TwoGOnly != null);

			// -- Modem Pin --
			setFieldValue('BTModem-Field-Modem-Pin', modem.Pin);

			// -- Modem CSD  --
			setFieldValue('BTModem-Field-Modem-CsdMode', modem.CsdMode);
			setFieldChecked('BTModem-Field-Modem-AutoConnect', modem.AutoConnect);
			setFieldValue('BTModem-Field-Modem-PhoneNumber', modem.PhoneNumber);
			setFieldValue('BTModem-Field-Modem-MaximumRedial', modem.MaximumRedial);

			// -- Modem GPRS --
			setFieldValue('BTModem-Field-Modem-InternetProtocol', modem.InternetProtocol);
			setFieldValue('BTModem-Field-Modem-AccessPointName', modem.AccessPointName);
			setFieldValue('BTModem-Field-Modem-Login', modem.Login);
			setFieldValue('BTModem-Field-Modem-Password', modem.Password);
			
			//Remove CSD Mode if ProFlex800
			if (ProFlex.is800()) {
				document.getElementById("BTModem-Label-Modem-Band").style.display = "none";
				document.getElementById("BTModem-Field-Modem-Band").style.display = "none";
				setItemDisplayed("BluetoothModem-CSD-Section",false);
			} else {
				document.getElementById("BTModem-Label-Modem-Band").style.display = "";
				document.getElementById("BTModem-Field-Modem-Band").style.display = "";
				setItemDisplayed("BluetoothModem-CSD-Section",true);
			}

			// Show form
			showConfigForm();			
		},

		failure : function (o) {
			if (!YAHOO.util.Connect.isCallInProgress(o)) {
				YAHOO.log("ERROR: Async call failed!");
			}

			// Retry
			setTimeout("initBluetoothModem()", 5000);
		},

		timeout : 10000,
		cache : false
    };

	// Make the call to the server for JSON data
	YAHOO.util.Connect.asyncRequest('GET', getConfigBlueToothModemURL, callbacks);

	// BT Secured Connection clicked
	YAHOO.util.Event.addListener('BTModem-Field-BT-Secured', 'click', onBtSecuredConnectionClick);

	// Field validation
	makeFieldValidator('BTModem-Field-BT-Pin', 'validating_BT-Pin');
	makeFieldValidator('BTModem-Field-Modem-PhoneNumber', 'validating_Phone');
	makeFieldValidator('BTModem-Field-Modem-MaximumRedial', 'validating_MaximumRedial');
}

function onBtSecuredConnectionClick(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	enableBtPin(isFieldChecked(field), field.value);
}

function enableBtPin(enable, value)
{
	setFieldVisible('BTModem-Field-BT-Pin', enable);
	
	if (enable)
	{
		// Restore Pin Code value or reset
		setFieldValue('BTModem-Field-BT-Pin', value == "-1" ? "" : value);
	}
	else
	{
		// Store Pin Code value
		setFieldValue('BTModem-Field-BT-Secured', getFieldValue('BTModem-Field-BT-Pin'));
		
		// Set to -1
		setFieldValue('BTModem-Field-BT-Pin', "-1");
	}
}