function initEmbeddedCasterSettings()
{
	// Define the callbacks for the asyncRequest
	var callbacks =
	{
		
		success : function (o)
		{

			YAHOO.log("RAW JSON DATA: " + o.responseText);

			// Process the JSON data returned from the server
			var data;
			try
			{
				data = YAHOO.lang.JSON.parse(o.responseText);
			}
			catch (x)
			{
				YAHOO.log("ERROR: JSON Parse failed!");

				// Retry
				setTimeout("initEmbeddedCasterSettings()", 5000);

				return;
			}

			YAHOO.log("PARSED DATA: " + YAHOO.lang.dump(data));
			
			var config = data.ConfigEmbeddedCasterSettings;
			
			//Options
			optionsGlobal = config.Options;
			
			setItemDisplayed('OptionC-OFF',(optionsGlobal.indexOf("C") == -1));
			setItemDisplayed('OptionC-ON',(optionsGlobal.indexOf("C") != -1));
			
			setFieldChecked('EmbeddedCasterSettings-Field-Activation', config.Activation );
			setFieldValue('EmbeddedCasterSettings-Field-Hostname', config.CasterHostname );
			setFieldValue('EmbeddedCasterSettings-Field-Port', config.CasterPortNumber );
			setFieldValue('EmbeddedCasterSettings-Field-Password', config.CasterPassword );
			
			setFieldValue('EmbeddedCasterSettings-Field-MaximumConnectionsPerUser', config.MaximumConnectionsPerUser);
			
			setFieldValue('EmbeddedCasterSettings-Field-CasterIdentifier', config.CasterIdentifier );
			setFieldValue('EmbeddedCasterSettings-Field-CasterOperator', config.CasterOperator );
			setFieldValue('EmbeddedCasterSettings-Field-Country', config.Country );
			setFieldValue('EmbeddedCasterSettings-Field-Latitude', config.Latitude );
			setFieldValue('EmbeddedCasterSettings-Field-Longitude', config.Longitude );
			setFieldValue('EmbeddedCasterSettings-Field-FallBackCasterAddressIP', config.FallBackCasterIPAddress );
			setFieldValue('EmbeddedCasterSettings-Field-FallBackCasterAddressPort', config.FallBackCasterPortNumber );
			setFieldValue('EmbeddedCasterSettings-Field-NetworkIdentifier', config.NetworkIdentifier );
			setFieldValue('EmbeddedCasterSettings-Field-NetworkOperator', config.NetworkOperator );
			setFieldChecked('EmbeddedCasterSettings-Field-Fee', config.Fee );
			setFieldValue('EmbeddedCasterSettings-Field-WebAddressNetworkInformation', config.WebAddressforNetworkInformation );
			setFieldValue('EmbeddedCasterSettings-Field-WebAddressStreamInformation', config.WebAddressforStreamInformation );
			setFieldValue('EmbeddedCasterSettings-Field-WebEmailAddressRegistration', config.WebEmailAddressforRegistration );
			
			setFieldPasswordVisible( 'EmbeddedCasterSettings-Field-Password', isFieldChecked( 'EmbeddedCasterSettings-Field-Password-Show' ) );
			
			
			// Show form
			showConfigForm();
		},

		failure : function (o)
		{
			if (!YAHOO.util.Connect.isCallInProgress(o))
			{
				YAHOO.log("ERROR: Async call failed!");
			}

			// Retry
			setTimeout("initEmbeddedCasterSettings()", 5000);
		},

		timeout : 10000,
		cache : false
    };
	
	// Make the call to the server for JSON data
	YAHOO.util.Connect.asyncRequest('GET', getConfigEmbeddedCasterSettings, callbacks);
	
	YAHOO.util.Event.addListener('EmbeddedCasterSettings-Field-Password-Show', 'click', onPasswordShowClick);
	
	makeFieldValidator('EmbeddedCasterSettings-Field-Port', 'validating_Port');

	makeFieldValidator('EmbeddedCasterSettings-Field-Country', 'validating_Country' );
	
	makeFieldValidator('EmbeddedCasterSettings-Field-Latitude', 'validating_Latitude' );
	makeFieldValidator('EmbeddedCasterSettings-Field-Longitude', 'validating_Longitude' );
	
	makeFieldValidator('EmbeddedCasterSettings-Field-FallBackCasterAddressPort', 'validating_Port');
	
	makeFieldValidator('EmbeddedCasterSettings-Field-Hostname', 'validating_SemiColon');
	//makeFieldValidator('EmbeddedCasterSettings-Field-Password', 'validating_SemiColon');
	
	makeFieldValidator('EmbeddedCasterSettings-Field-MaximumConnectionsPerUser', 'validating_MaximumConnectionsPerUser');
	
	makeFieldValidator('EmbeddedCasterSettings-Field-CasterIdentifier', 'validating_SemiColon');
	makeFieldValidator('EmbeddedCasterSettings-Field-CasterOperator', 'validating_SemiColon');
	makeFieldValidator('EmbeddedCasterSettings-Field-FallBackCasterAddressIP', 'validating_SemiColon');
	makeFieldValidator('EmbeddedCasterSettings-Field-NetworkIdentifier', 'validating_SemiColon');
	makeFieldValidator('EmbeddedCasterSettings-Field-NetworkOperator', 'validating_SemiColon');
	makeFieldValidator('EmbeddedCasterSettings-Field-WebAddressNetworkInformation', 'validating_SemiColon');
	makeFieldValidator('EmbeddedCasterSettings-Field-WebAddressStreamInformation', 'validating_SemiColon');
	makeFieldValidator('EmbeddedCasterSettings-Field-WebEmailAddressRegistration', 'validating_SemiColon');
}

function onPasswordShowClick( ev )
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	setFieldPasswordVisible('EmbeddedCasterSettings-Field-Password', isFieldChecked(field));
}

function validating_MaximumConnectionsPerUser( ev, provider )
{
	var field = YAHOO.util.Event.getTarget(ev);
	var valid = !isNaN(field.value) && field.value > 0 && field.value <= 100;
	
	notifyErrorProvider(provider, !valid);
}

function validating_Port(ev, provider)
{
	var field = YAHOO.util.Event.getTarget(ev);
	var valid = !isNaN(field.value) && isValidInteger(field.value) && field.value >= 0 && field.value <= 65535;
	
	notifyErrorProvider(provider, !valid);
}

function validating_Latitude(ev, provider)
{
	var field = YAHOO.util.Event.getTarget(ev);
	var valid = !isNaN(field.value) && field.value >= -90 && field.value <= 90 
		&& (field.value.indexOf('.') == -1 || (field.value.length - field.value.indexOf('.') - 1 <= 2));
	
	notifyErrorProvider(provider, !valid);
}

function validating_Longitude(ev, provider)
{
	var field = YAHOO.util.Event.getTarget(ev);
	var valid = !isNaN(field.value) && field.value >= -180 && field.value <= 180
		&& (field.value.indexOf('.') == -1 || (field.value.length - field.value.indexOf('.') - 1 <= 2));
	
	notifyErrorProvider(provider, !valid);
}

function validating_Country( ev, provider )
{
	var field = YAHOO.util.Event.getTarget(ev);
	var regexp = new RegExp("^[A-Z]{3}$");
	var valid = regexp.test( field.value );
	
	notifyErrorProvider(provider, !valid);
}