var configGlobal;

function initNtripServerSetup()
{
	// Define the callbacks for the asyncRequest
	var callbacks = {
		
		success : function (o)
		{

			YAHOO.log("RAW JSON DATA: " + o.responseText);

			// Process the JSON data returned from the server
			var data;
			try
			{
				data = YAHOO.lang.JSON.parse(o.responseText);
			}
			catch (x)
			{
				YAHOO.log("ERROR: JSON Parse failed!");

				// Retry
				setTimeout("initNtripServerSetup()", 5000);

				return;
			}

			YAHOO.log("PARSED DATA: " + YAHOO.lang.dump(data));

			// Get config
			var config = data.ConfigFullSetup;
			if (config == null) return;

			// Check Mode
			if (config.Mode != null)
			{	
				if (config.Mode == 1)
				{
					//TODO: warning, you are in rover mode!
				}
			}
			
			//Remove CSD Mode if ProFlex800
			var network1ReceiverModel = document.getElementById('NtripServer-Field-Network1-ReceiverModel');
			if (ProFlex.is800()) {
				network1ReceiverModel.value = 800;
			} else {
				network1ReceiverModel.value = 500;
			}
			
			processBaseSetup( config );
			processStream1DiffMessage( config );
			processStream2DiffMessage( config );
			//processNtripParameters( config );
			
			// -- Mount Point List --
			var names = new Array();
			var fullnames = new Array();
			for (i in config.Caster.MountPoints)
			{	
				names.push(config.Caster.MountPoints[i].Name);
				fullnames.push(config.Caster.MountPoints[i].Name + ' (' + config.Caster.MountPoints[i].Identifier + ')');
			}
				
			setFieldOptions( 'NtripServer-Field-Network1-Ntrip-MountPointsView', fullnames, names);
			setFieldOptions( 'NtripServer-Field-Network2-Ntrip-MountPointsView', fullnames, names);
			
			setFieldValue( 'NtripServer-Field-Network1-Ntrip-MountStorage', config.Network1.NtripPortP.Mount );
			setFieldValue( 'NtripServer-Field-Network2-Ntrip-MountStorage', config.Network2.NtripPortQ.Mount );

			setFieldValue('NtripServer-Field-Network1-Ntrip-MountPointsView', config.Network1.NtripPortP.Mount);
			setFieldValue('NtripServer-Field-Network2-Ntrip-MountPointsView', config.Network2.NtripPortQ.Mount);
			

			setFieldPasswordVisible( 'NtripServer-Field-Network2-Ntrip-Password', isFieldChecked( 'NtripServer-Field-Network2-Ntrip-Password-Show' ) );
			
			//Fix NTRIP Server, there is no "None" in Network 1
			if (config.Network1.ConnectionType == 0)
				config.Network1.ConnectionType = 3;
			
			
			// -- Option [C] --
			if (optionsGlobal.indexOf("C") == -1) {
				var network1ConnectionType = document.getElementById('NtripServer-Field-Network1-ConnectionType');
				network1ConnectionType.remove(2);
				
				var network2ConnectionType = document.getElementById('NtripServer-Field-Network2-ConnectionType');
				network2ConnectionType.remove(2);
				
				if (config.Network1.ConnectionType == 7) {
					config.Network1.ConnectionType = 3;
					
					if (config.Network1.NtripPortE.AddressIp == "localhost" ||
						config.Network1.NtripPortE.AddressIp == "127.0.0.1")
						config.Network1.NtripPortE.AddressIp = "0.0.0.0";
					
					if (config.Network1.NtripPortP.AddressIp == "localhost" ||
						config.Network1.NtripPortP.AddressIp == "127.0.0.1")
						config.Network1.NtripPortP.AddressIp = "0.0.0.0";
				} else {
					//config.Network1.NtripPortE.AddressIp = "";
					//config.Network1.NtripPortP.AddressIp = "";
				}
				
				if (config.Network2.ConnectionType == 7) {
					config.Network2.ConnectionType = 0;
					
					if (config.Network2.NtripPortQ.AddressIp == "localhost" ||
						config.Network2.NtripPortQ.AddressIp == "127.0.0.1")
						config.Network2.NtripPortQ.AddressIp = "0.0.0.0";
				} else {
					//config.Network2.NtripPortQ.AddressIp = "";
				}
			}
			
			configGlobal = config;
			
			//Options
			optionsGlobal = config.Options;
			
			// Ntrip Server 1
			setFieldValue('NtripServer-Field-Network1-ConnectionType', config.Network1.ConnectionType);
			setFieldChecked('NtripServer-Field-Network1-ConnectNow', 1); // Always checked by default
			enableNetwork1Parameters(config.Network1.ConnectionType);

			//Ntrip Server 2
			setFieldValue('NtripServer-Field-Network2-ConnectionType', config.Network2.ConnectionType);
			setFieldChecked('NtripServer-Field-Network2-ConnectNow', 1); // Always checked by default			
			enableNetwork2Parameters(config.Network2.ConnectionType);
			
			
			// Show form
			showConfigForm();
		},
		failure : function (o)
		{
			if (!YAHOO.util.Connect.isCallInProgress(o))
			{
				YAHOO.log("ERROR: Async call failed!");
			}

			// Retry
			setTimeout("initNtripServerSetup()", 5000);
		},

		timeout : 10000,
		cache : false
    };

	//Mount POST
	YAHOO.util.Event.addListener('NtripServer-Field-Network1-Ntrip-MountPointsView', 'change', onNetwork1MountChanged);
	YAHOO.util.Event.addListener('NtripServer-Field-Network1-Ntrip-Mount', 'change', onNetwork1MountChanged);
	YAHOO.util.Event.addListener('NtripServer-Field-Network2-Ntrip-MountPointsView', 'change', onNetwork2MountChanged);
	YAHOO.util.Event.addListener('NtripServer-Field-Network2-Ntrip-Mount', 'change', onNetwork2MountChanged);
	
	
	// Show Characters
	YAHOO.util.Event.addListener('NtripServer-Field-Network1-Ntrip-Password-Show', 'click', onNetwork1PasswordShowClick);
	YAHOO.util.Event.addListener('NtripServer-Field-Network2-Ntrip-Password-Show', 'click', onNetwork2PasswordShowClick);
	
	// Make the call to the server for JSON data
	YAHOO.util.Connect.asyncRequest('GET', getConfigFullSetupURL, callbacks);
	
	YAHOO.util.Event.addListener('NtripServer-Field-Network1-ConnectionType', 'change', onNetwork1ConnectionTypeChanged);
	YAHOO.util.Event.addListener('NtripServer-Field-Network2-ConnectionType', 'change', onNetwork2ConnectionTypeChanged);
	
	YAHOO.util.Event.addListener('NtripServer-Field-Stream1-MessageType', 'change', onDiffMessageChanged );
	YAHOO.util.Event.addListener('NtripServer-Field-Stream2-MessageType', 'change', onDiffMessageChanged );
	
	//Tooltip
	YAHOO.util.Event.addListener('NtripServer-Label-Stream1-Tooltip', 'mouseover', onTooltipOver);
	YAHOO.util.Event.addListener('NtripServer-Label-Stream2-Tooltip', 'mouseover', onTooltipOver);
	
	setBaseSetupValidators();
	setNtripValidators();
	//initFieldValidator();
}

function onNetwork1MountChanged( ev )
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	setFieldValue( 'NtripServer-Field-Network1-Ntrip-MountStorage', field.value );
}

function onNetwork2MountChanged( ev )
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	setFieldValue( 'NtripServer-Field-Network2-Ntrip-MountStorage', field.value );
}

function onNetwork1PasswordShowClick( ev )
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	setFieldPasswordVisible('NtripServer-Field-Network1-Ntrip-Password', isFieldChecked(field));
}

function onNetwork2PasswordShowClick( ev )
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	setFieldPasswordVisible('NtripServer-Field-Network2-Ntrip-Password', isFieldChecked(field));
}

function onNetwork1ConnectionTypeChanged(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	enableNetwork1Parameters(field.value);
}

function onNetwork2ConnectionTypeChanged(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	enableNetwork2Parameters(field.value);
}

function enableNetwork1Parameters(connection)
{
	switch (connection)
	{
	case 3:
	case "3":
		// -- Network1 / Ntrip Port E --
		setFieldValue('NtripServer-Field-Network1-Ntrip-AddressIP', configGlobal.Network1.NtripPortE.AddressIp);
		setFieldValue('NtripServer-Field-Network1-Ntrip-AddressPort', configGlobal.Network1.NtripPortE.AddressPort);
		setFieldValue('NtripServer-Field-Network1-Ntrip-Mount', configGlobal.Network1.NtripPortE.Mount);
		setFieldValue('NtripServer-Field-Network1-Ntrip-Login', configGlobal.Network1.NtripPortE.Login);
		setFieldValue('NtripServer-Field-Network1-Ntrip-Password', configGlobal.Network1.NtripPortE.Password);
		setFieldPasswordVisible('NtripServer-Field-Network1-Ntrip-Password', false);
		setFieldValue( 'NtripServer-Field-Network1-Ntrip-MountStorage', configGlobal.Network1.NtripPortP.Mount );
	break;
	case 5:
	case "5":
		// -- Network1 / Ntrip Port P --
		setFieldValue('NtripServer-Field-Network1-Ntrip-AddressIP', configGlobal.Network1.NtripPortP.AddressIp);
		setFieldValue('NtripServer-Field-Network1-Ntrip-AddressPort', configGlobal.Network1.NtripPortP.AddressPort);
		setFieldValue('NtripServer-Field-Network1-Ntrip-Mount', configGlobal.Network1.NtripPortP.Mount);
		setFieldValue('NtripServer-Field-Network1-Ntrip-Login', configGlobal.Network1.NtripPortP.Login);
		setFieldValue('NtripServer-Field-Network1-Ntrip-Password', configGlobal.Network1.NtripPortP.Password);
		setFieldPasswordVisible('NtripServer-Field-Network1-Ntrip-Password', false);
		setFieldValue( 'NtripServer-Field-Network1-Ntrip-MountStorage', configGlobal.Network1.NtripPortP.Mount );
	break;
	case 7:
	case "7":
		// -- Network1 / Ntrip (Embedded) Port P --
	    setFieldValue('NtripServer-Field-Network1-Ntrip-AddressIP', configGlobal.Caster.AddressIP);
	    setFieldValue('NtripServer-Field-Network1-Ntrip-AddressPort', configGlobal.Caster.AddressPort);
	    setFieldValue('NtripServer-Field-Network1-Ntrip-Password', configGlobal.Caster.Password);

		setFieldPasswordVisible( 'NtripServer-Field-Network1-Ntrip-Password', isFieldChecked( 'NtripServer-Field-Network1-Ntrip-Password-Show' ) );
	
		var select = document.getElementById('NtripServer-Field-Network1-Ntrip-MountPointsView');
		if (select.selectedIndex == -1)
			select.selectedIndex = 0;
		if (select.options.length > 0) {
			var selectedValue = select.options[select.selectedIndex].value;
			setFieldValue('NtripServer-Field-Network1-Ntrip-MountStorage', selectedValue );
		} else {
			setFieldValue('NtripServer-Field-Network1-Ntrip-MountStorage', "" );
		}
	break;
	}
	
	setItemDisplayed('NtripServer-Group-Network1-Ntrip', true);

	setFieldVisible('NtripServer-Field-Network1-ConnectNow', true);
	
	setFieldVisible( 'NtripServer-Field-Network1-Ntrip-Mount', connection == 3 || connection == 5 || connection == 7);
	setDisplayElement( 'NtripServer-Field-Network1-Ntrip-Mount', connection == 3 || connection == 5);
	setDisplayElement( 'NtripServer-Field-Network1-Ntrip-MountPointsView', connection == 7);
	
	setFieldVisible( 'NtripServer-Field-Network1-Ntrip-Password-Show', connection == 7);
	
    setReadOnly('NtripServer-Field-Network1-Ntrip-AddressIP', connection == 7);
    setReadOnly('NtripServer-Field-Network1-Ntrip-AddressPort', connection == 7);
    setReadOnly('NtripServer-Field-Network1-Ntrip-Password', connection == 7);
    
    //alert("Hidden Mount Field value = " + document.getElementById('NtripServer-Field-Network1-Ntrip-MountStorage').value);
}

function enableNetwork2Parameters(connection)
{
	switch (connection)
	{
	case 5:
	case "5":
		// -- Network2 / Ntrip Port Q --
		setFieldValue('NtripServer-Field-Network2-Ntrip-AddressIP', configGlobal.Network2.NtripPortQ.AddressIp);
		setFieldValue('NtripServer-Field-Network2-Ntrip-AddressPort', configGlobal.Network2.NtripPortQ.AddressPort);
		setFieldValue('NtripServer-Field-Network2-Ntrip-Mount', configGlobal.Network2.NtripPortQ.Mount);
		setFieldValue('NtripServer-Field-Network2-Ntrip-Login', configGlobal.Network2.NtripPortQ.Login);
		setFieldValue('NtripServer-Field-Network2-Ntrip-Password', configGlobal.Network2.NtripPortQ.Password);
		setFieldPasswordVisible('NtripServer-Field-Network2-Ntrip-Password', false);
		setFieldValue( 'NtripServer-Field-Network2-Ntrip-MountStorage', configGlobal.Network2.NtripPortQ.Mount );
	break;
	case 7:
	case "7":
		// -- Network2 / Ntrip (Embedded) Port Q --
	    setFieldValue('NtripServer-Field-Network2-Ntrip-AddressIP', configGlobal.Caster.AddressIP);
	    setFieldValue('NtripServer-Field-Network2-Ntrip-AddressPort', configGlobal.Caster.AddressPort);
	    setFieldValue('NtripServer-Field-Network2-Ntrip-Password', configGlobal.Caster.Password);
	    setFieldPasswordVisible( 'NtripServer-Field-Network2-Ntrip-Password', isFieldChecked( 'NtripServer-Field-Network2-Ntrip-Password-Show' ) );
	
	    var select = document.getElementById('NtripServer-Field-Network2-Ntrip-MountPointsView');
		if (select.selectedIndex == -1)
			select.selectedIndex = 0;
		if (select.options.length > 0) {
			var selectedValue = select.options[select.selectedIndex].value;
			setFieldValue('NtripServer-Field-Network2-Ntrip-MountStorage', selectedValue );
		} else {
			setFieldValue('NtripServer-Field-Network2-Ntrip-MountStorage', "" );
		}
	break;
	}
	
	setItemDisplayed('NtripServer-Group-Network2-Ntrip',
		connection == 5 /* Ethernet Ntrip server */ ||
		connection == 7 /* Embedded Ntrip Caster */);

	setFieldVisible('NtripServer-Field-Network2-ConnectNow', connection > 0);
	
	setFieldVisible( 'NtripServer-Field-Network2-Ntrip-Mount', connection == 5 || connection == 7);
	setDisplayElement( 'NtripServer-Field-Network2-Ntrip-Mount', connection == 5);
	setDisplayElement( 'NtripServer-Field-Network2-Ntrip-MountPointsView', connection == 7);
	
	setFieldVisible( 'NtripServer-Field-Network2-Ntrip-Password-Show', connection == 7);
	
    setReadOnly('NtripServer-Field-Network2-Ntrip-AddressIP', connection == 7);
    setReadOnly('NtripServer-Field-Network2-Ntrip-AddressPort', connection == 7);
    setReadOnly('NtripServer-Field-Network2-Ntrip-Password', connection == 7);
}