var internalRadioTypeSelect = null;
var configGlobal;

function initRoverSetup()
{
	// Define the callbacks for the asyncRequest
	var callbacks = {
		
		success : function (o) {

			YAHOO.log("RAW JSON DATA: " + o.responseText);

			// -- Help link
			if (ProFlex.isDuo()) {
                setItemDisplayed('Index-Help', false);
				setItemDisplayed('Index-Help-HDS', true);
			} else {
				setItemDisplayed('Index-Help-HDS', false);
				setItemDisplayed('Index-Help', true);
			}

			// Process the JSON data returned from the server
			var data;
			try {
				data = YAHOO.lang.JSON.parse(o.responseText);
			}
			catch (x) {
				YAHOO.log("ERROR: JSON Parse failed!");

				// Retry
				setTimeout("initRoverSetup()", 5000);

				return;
			}

			YAHOO.log("PARSED DATA: " + YAHOO.lang.dump(data));

			// Get config
			var config = data.ConfigRoverSetup;
			if (config == null) return;
			
			loadAntennaInfo(config.Antenna.List);

			//Remove CSD Mode if ProFlex800
			var network1ReceiverModel = document.getElementById('RoverSetup-Field-Network-ReceiverModel');
			if (ProFlex.is800()) {
				var elSel = document.getElementById('RoverSetup-Field-Network-ConnectionType');
				elSel.remove(1);
				network1ReceiverModel.value = 800;
			} else {
				network1ReceiverModel.value = 500;
			}
			
			configGlobal = config;
			
			// -- Rover Mode (HDS800) --
   			//selectValueElseDefault('RoverSetup-Field-Mode', config.Mode, 1);
			//setFieldVisible('RoverSetup-Field-Mode', ProFlex.isDuo());
			var mode = 1;
			if (ProFlex.isDuo()) {
			    setItemDisplayed('RoverSetup-Group-Mode',true);
			    if (config.Mode != 0) mode = config.Mode;
			}
		    setFieldValue('RoverSetup-Field-Mode', mode);

			// -- Ambiguity Fixing --
			setFieldValue('RoverSetup-Field-AmbiguityFixing', 
				config.AmbiguityFixing != null && config.AmbiguityFixing > 0 ? 
					config.AmbiguityFixing.toFixed(1) : "0");
				
			// -- Fast RTK --
			setFieldChecked('RoverSetup-Field-FastRTK', config.FastRtk);

			// -- Moving Mode --
			setFieldVisible('RoverSetup-Field-MovingMode', (mode != 2));
		    setFieldChecked('RoverSetup-Field-MovingMode', config.MovingMode);
		    enableMovingBase(config.MovingMode);
			
			//Dynamic. Not sure that it is only single parameter
			setFieldValue( 'RoverSetup-Field-Dynamic', config.Dynamic );
			
			// -- Antenna List --
			if (config.Antenna.List != null)
			{
				var names = new Array();
				var namesCpt;
				
				for (i in config.Antenna.List)
				{	
				    if (config.Antenna.List[i].AntennaName != null)
				    {
				        nameCpt = names.push(config.Antenna.List[i].AntennaName);
				    }
				}
				
				setFieldOptions('RoverSetup-Field-OwnAntennaName', names, names);
				addFieldOptions('RoverSetup-Field-OutAntennaName', names, names); // (Keep "OFF" option and add others)
				addFieldOptions('RoverSetup-Field-Antenna2Name', names, names);
			}

			// -- Own Antenna Name --
			setFieldValue('RoverSetup-Field-OwnAntennaName', config.Antenna.OwnAntennaName);

			// -- Out Antenna Name --
			setFieldValue('RoverSetup-Field-OutAntennaName', config.Antenna.OutAntennaName);

			// -- Antenna 2 Name --
			setFieldValue('RoverSetup-Field-Antenna2Name', config.Antenna.Antenna2Name);
			setFieldVisible('RoverSetup-Field-Antenna2Name', ProFlex.isDuo());
			if (!ProFlex.isDuo())
			    removeFieldFromPost('RoverSetup-Field-Antenna2Name');
						
			// -- Antenna Height --
			setFieldValueAsDistance('RoverSetup-Field-AntennaHeight', config.Antenna.AntennaHeight);
			setFieldValueAsDistance('RoverSetup-Field-AntennaHeight2', config.Antenna.AntennaHeight2);

			// -- Antenna Height Measurement Type --
			setFieldValue('RoverSetup-Field-MeasurementType', config.Antenna.MeasurementType);
			enableSHMP(config.Antenna.MeasurementType);

			setFieldValue('RoverSetup-Field-MeasurementType2', config.Antenna.MeasurementType2);
			enableSHMP2(config.Antenna.MeasurementType2);

			// -- Antenna Radius --
			setFieldValueAsDistance('RoverSetup-Field-AntennaRadius', config.Antenna.AntennaRadius);
			setFieldValueAsDistance('RoverSetup-Field-AntennaRadius2', config.Antenna.AntennaRadius2);

			// -- Antenna Vertical Offset --
			setFieldValueAsDistance('RoverSetup-Field-AntennaSHMP', config.Antenna.AntennaVerticalOffset);
			setFieldValueAsDistance('RoverSetup-Field-AntennaSHMP2', config.Antenna.AntennaVerticalOffset2);
			
			// -- Antenna Reduction Mode --
			setFieldValue('RoverSetup-Field-AntennaReductionMode', config.Antenna.AntennaReductionMode);

			// -- Position Elevation Mask --
			setFieldValue('RoverSetup-Field-ElevationMask', config.PositionElevationMask);

			if (ProFlex.isDuo()) 
			{
			    setItemDisplayed('RoverSetup-Group-Antenna2', true);
			    setItemText('RoverSetup-Antenna2',getResource('RoverSetup-Field-Antenna2Name-Trans'));
			    setItemText('RoverSetup-Label-AntennaHeight2',getResource('RoverSetup-Field-AntennaHeight-Trans'));
			    setItemText('RoverSetup-Label-AntennaRadius2',getResource('RoverSetup-Field-AntennaRadius-Trans'));
			    setItemText('RoverSetup-Label-AntennaSHMP2',getResource('RoverSetup-Field-AntennaSHMP-Trans'));
			    setItemText('RoverSetup-Label-MeasurementType2',getResource('RoverSetup-Field-MeasurementType-Trans'));
			    setItemText('RoverSetup-Field-MeasurementType2-0',getResource('RoverSetup-Field-MeasurementType|0-Trans'));
			    setItemText('RoverSetup-Field-MeasurementType2-1',getResource('RoverSetup-Field-MeasurementType|1-Trans'));
		    }		        

	        //Options
	        optionsGlobal = config.Options;

	        // -- GPS --
	        setFieldChecked('RoverSetup-Field-GPS', config.SatelliteGPS);
        	
	        // -- GLONASS --
			//setFieldChecked('RoverSetup-Field-GLONASS', config.SatelliteGlonass);
	        if (optionsGlobal.indexOf("S") != -1)
	        {
	            setFieldVisible('RoverSetup-Field-GLONASS', 1);
	            setFieldChecked('RoverSetup-Field-GLONASS', config.SatelliteGlonass);
	        } else {
	            setFieldVisible('RoverSetup-Field-GLONASS', 0);
	        }
        	
	        // -- SBAS --
			setFieldChecked('RoverSetup-Field-SBAS', config.SatelliteSbas);

	        // -- GALILEO --
	        //setFieldChecked('RoverSetup-Field-GALILEO', config.SatelliteGALILEO);
	        if (optionsGlobal.indexOf("O") != -1)
	        {
	            setFieldVisible('RoverSetup-Field-GALILEO', 1);
	            setFieldChecked('RoverSetup-Field-GALILEO', config.SatelliteGALILEO);
	        } else {
	            setFieldVisible('RoverSetup-Field-GALILEO', 0);
	        }

	        // -- QZSS --
	        setFieldChecked('RoverSetup-Field-QZSS', config.SatelliteQZSS);
	
	        // -- Serial Port A --
			selectValueElseDefault('RoverSetup-Field-PortA-ConnectionType', config.SerialPortA.ConnectionType, 0); // If selected radio not allowed here then select None
			setFieldValue('RoverSetup-Field-PortA-BaudRate', config.SerialPortA.BaudRate);
			setFieldChecked('RoverSetup-Field-PortA-RtsCts', config.SerialPortA.RtsCts);
			setFieldValue('RoverSetup-Field-PortA-Mode', config.SerialPortA.Mode);
			//enablePortAParameters(config.SerialPortA.ConnectionType);// Do not call: better to show real status
			
			enablePortARtsCts( config.SerialPortA.Mode );
			
			// -- Serial Port B --
			selectValueElseDefault('RoverSetup-Field-PortB-ConnectionType', config.SerialPortB.ConnectionType, 0); // If selected radio not allowed here then select None
			setFieldValue('RoverSetup-Field-PortB-BaudRate', config.SerialPortB.BaudRate);
			setFieldChecked('RoverSetup-Field-PortB-RtsCts', config.SerialPortB.RtsCts);
			//enablePortBParameters(config.SerialPortB.ConnectionType);// Do not call: better to show real status

			// -- Serial Port F --
			selectValueElseDefault('RoverSetup-Field-PortF-ConnectionType', config.SerialPortF.ConnectionType, 0); // If selected radio not allowed here then select None
			setFieldValue('RoverSetup-Field-PortF-BaudRate', config.SerialPortF.BaudRate);
			setFieldChecked('RoverSetup-Field-PortF-RtsCts', config.SerialPortF.RtsCts);
			//enablePortFParameters(config.SerialPortF.ConnectionType);// Do not call: better to show real status

			// -- Internal Radio Port D --
			setFieldValue('RoverSetup-Field-PortD-ConnectionType', config.InternalRadioD.ConnectionType);
			enableField('RoverSetup-Field-PortD-ConnectionType', false);
			checkRadioButton(
				new Array(
					'RoverSetup-Field-Radio-Power-On', 
					'RoverSetup-Field-Radio-Power-Off'), config.InternalRadioD.Power);
			enablePortDParameters(config.InternalRadioD.ConnectionType);
			
			// -- Show only current radio type --
			if (internalRadioTypeSelect == null) internalRadioTypeSelect = new SelectFieldManager('RoverSetup-Field-PortD-ConnectionType');
			internalRadioTypeSelect.setFieldOptions([config.InternalRadioD.ConnectionType]);
			

			// -- Network --
			setFieldValue('RoverSetup-Field-Network-ConnectionType', config.Network.ConnectionType);
			setFieldChecked('RoverSetup-Field-Network-ConnectNow', 1 /*config.Network.Connected*/); // Always checked by default
			setFieldValue('RoverSetup-Field-Network-PhoneNumber', config.Network.ModemPhoneNumber);
			enableNetworkParameters(config.Network.ConnectionType);

			// -- Differential Port --
			checkRadioButton(
				new Array(
					'RoverSetup-Field-DifferentialPortMode-Auto', 
					'RoverSetup-Field-DifferentialPortMode-Manual'), config.DifferentialPortMode);
			setFieldValue('RoverSetup-Field-Stream1-Port', config.DifferentialStream1Selected);
			setFieldValue('RoverSetup-Field-Stream2-Port', config.DifferentialStream2Selected);
			enableDifferentialStreams(config.DifferentialPortMode);

			// -- Hot Standby RTK --
			setItemDisplayed('RoverSetup-Group-HotStandbyRTK', (mode < 2));
			setFieldChecked('RoverSetup-Field-HotStandbyRTK-HotStandbyRTK', config.HotStandbyRTK.HotStandbyRTK);
			setFieldValue('RoverSetup-Field-HotStandbyRTK-Stream-Port', config.HotStandbyRTK.StreamSelected);
			enableHotStandbyRTKParameters(config.HotStandbyRTK.HotStandbyRTK);

			// Show form
			showConfigForm();
		},

		failure : function (o) {
			if (!YAHOO.util.Connect.isCallInProgress(o)) {
				YAHOO.log("ERROR: Async call failed!");
			}

			// Retry
			setTimeout("initRoverSetup()", 5000);
		},

		timeout : 10000,
		cache : false
    };

	// Make the call to the server for JSON data
	YAHOO.util.Connect.asyncRequest('GET', getConfigRoverSetupURL, callbacks);
	
	// Mode changed
	YAHOO.util.Event.addListener('RoverSetup-Field-Mode', 'change', onModeChanged);
	
	// Units changed
	YAHOO.util.Event.addListener('distances', 'change', onDistanceUnitChanged);

	// Fast RTK / Moving Mode changed
	YAHOO.util.Event.addListener('RoverSetup-Field-FastRTK', 'click', onFastRtkClicked);
	YAHOO.util.Event.addListener('RoverSetup-Field-MovingMode', 'click', onMovingModeClicked);

	// Measurement Type changed
	YAHOO.util.Event.addListener('RoverSetup-Field-MeasurementType', 'change', onMeasurementTypeChanged);
	YAHOO.util.Event.addListener('RoverSetup-Field-MeasurementType2', 'change', onMeasurementTypeChanged2);
	
	// ConnectionType changed
	YAHOO.util.Event.addListener('RoverSetup-Field-PortA-ConnectionType', 'change', onPortAConnectionTypeChanged);
	YAHOO.util.Event.addListener('RoverSetup-Field-PortB-ConnectionType', 'change', onPortBConnectionTypeChanged);
	YAHOO.util.Event.addListener('RoverSetup-Field-PortF-ConnectionType', 'change', onPortFConnectionTypeChanged);
	YAHOO.util.Event.addListener('RoverSetup-Field-Network-ConnectionType', 'change', onNetworkConnectionTypeChanged);

	// Port mode changed
	YAHOO.util.Event.addListener('RoverSetup-Field-PortA-Mode', 'change', onPortAModeChanged);
	
	// Ntrip Host changed
	YAHOO.util.Event.addListener('RoverSetup-Field-Network-Ntrip-AddressIP', 'change', onNtripHostChange);
	YAHOO.util.Event.addListener('RoverSetup-Field-Network-Ntrip-AddressPort', 'change', onNtripHostChange);
	
	// Ntrip Mount changed
	YAHOO.util.Event.addListener('RoverSetup-Field-Network-Ntrip-Mount', 'change', onNtripMountChange);
	
	// Ntrip Source Table load click
	YAHOO.util.Event.addListener('RoverSetup-Button-LoadSourceTable', 'click', onLoadSourceTable);
	
	// DifferentialPortMode changed
	YAHOO.util.Event.addListener('RoverSetup-Field-DifferentialPortMode-Auto', 'click', onDifferentialPortModeChanged);
	YAHOO.util.Event.addListener('RoverSetup-Field-DifferentialPortMode-Manual', 'click', onDifferentialPortModeChanged);
	
	// HotStandbyRTK clicked
	YAHOO.util.Event.addListener('RoverSetup-Field-HotStandbyRTK-HotStandbyRTK', 'click', onHotStandbyRTKClicked);

    // GPS and GLONASS Check Box			
	YAHOO.util.Event.addListener('RoverSetup-Field-GPS', 'click', onFieldGPSChange );
	YAHOO.util.Event.addListener('RoverSetup-Field-GLONASS', 'click', onFieldGLONASSChange );

	// Field validation	
	makeFieldValidator('RoverSetup-Field-Network-PhoneNumber', 'validating_Phone');
	makeFieldValidator('RoverSetup-Field-Network-DirectIP-AddressIP', 'validating_Host');
	makeFieldValidator('RoverSetup-Field-Network-DirectIP-AddressPort', 'validating_Port');
	makeFieldValidator('RoverSetup-Field-Network-Ntrip-AddressIP', 'validating_Host');
	makeFieldValidator('RoverSetup-Field-Network-Ntrip-AddressPort', 'validating_Port');
	makeFieldValidator('RoverSetup-Field-ElevationMask', 'validating_ElevationMask');
	
	makeFieldValidator('RoverSetup-Field-AntennaHeight', 'validating_AntennaHeight');
	makeFieldValidator('RoverSetup-Field-AntennaRadius', 'validating_ntennaRadius');
	makeFieldValidator('RoverSetup-Field-AntennaSHMP', 'validating_AntennaSHMP');

	makeFieldValidator('RoverSetup-Field-AntennaHeight2', 'validating_AntennaHeight');
	makeFieldValidator('RoverSetup-Field-AntennaRadius2', 'validating_ntennaRadius');
	makeFieldValidator('RoverSetup-Field-AntennaSHMP2', 'validating_AntennaSHMP');
}

function getPrefixName()
{
	return "RoverSetup";
}

function NtripSourceTableView(source, loading)
{
	var myColumnDefs = [
		{key:"MountPoint", sortable:true},
		{key:"StreamName", sortable:true},
		{key:"Format", sortable:true},
		{key:"System", sortable:true},
		{key:"Country", sortable:true},
		{key:"Latitude", sortable:true},
		{key:"Longitude", sortable:true},
		{key:"NMEA", formatter:NtripSourceTableView.formatNmea, sortable:true},
		{key:"Solution", formatter:NtripSourceTableView.formatSolution, sortable:true},
		{key:"Generator", sortable:true}
	];

	this.selectMountPoint = function(name) 
	{
	    // Get source table records
	    var records = this.ntripSourceDataTable.getRecordSet().getRecords();
    		
	    // Unselect all rows
	    this.ntripSourceDataTable.unselectAllRows();
    		
	    // Search for mount point
	    //for ( var i in records)
	    for (var i=0; i<records.length; i++)
	    {
		    var record = records[i];
		    if (record.getData().MountPoint == name)
		    {
			    // Select row for selected mount point
			    this.ntripSourceDataTable.selectRow(record);
			    return true;
		    }
	    }
		return false;
	};
	
	this.getSelectedMountPoint = function()
	{
		// Get selection
		var rows = this.ntripSourceDataTable.getSelectedRows();
		if (rows.length == 1)
		{
			// Get record
			var record = this.ntripSourceDataTable.getRecord(rows[0]);
			
			// Get mount point
			var mount = record.getData();
			
			// Return
			return mount;
		}
		
		return null;
	};
	
	this.destroy = function()
	{
		this.ntripSourceDataTable.destroy();
		this.ntripSourceDataTable = null;			
	};
		
	// Localized columns
	for (col in myColumnDefs)
	{
		myColumnDefs[col].label = getResource("NtripSourceTable-Resource-" + myColumnDefs[col].key);
	}

	// Data source
	this.myDataSource = new YAHOO.util.DataSource(source);
	this.myDataSource.responseType = YAHOO.util.DataSource.TYPE_JSON;
	this.myDataSource.responseSchema = {
		resultsList: "NtripSourceTable",
		fields: ["MountPoint","StreamName","Format","System","Country","Latitude","Longitude","NMEA","Solution","Generator"]
	};
	
	// Table configs
	var configs = {
		selectionMode:"single",
		width:"600px",
		height:"400px"
	};
	
	// Localized messages
	configs.MSG_EMPTY = getResource(loading ? 
		'NtripSourceTable-Resource-Loading' : 
		'NtripSourceTable-Resource-Empty');
	configs.MSG_LOADING = getResource('NtripSourceTable-Resource-Loading');
	configs.MSG_SORTASC = getResource('NtripSourceTable-Resource-SortAsc');
	configs.MSG_SORTDESC = getResource('NtripSourceTable-Resource-SortDesc');
	
	// Create table
	this.ntripSourceDataTable = new YAHOO.widget.ScrollingDataTable("NtripSourceTableView",
			myColumnDefs, this.myDataSource, configs);
			
	// Subscribe to events for row selection
	this.ntripSourceDataTable.subscribe("rowMouseoverEvent", this.ntripSourceDataTable.onEventHighlightRow);
	this.ntripSourceDataTable.subscribe("rowMouseoutEvent", this.ntripSourceDataTable.onEventUnhighlightRow);
	this.ntripSourceDataTable.subscribe("rowClickEvent", this.ntripSourceDataTable.onEventSelectRow);

	// Programmatically bring focus to the instance so arrow selection works immediately
	this.ntripSourceDataTable.focus();
}

NtripSourceTableView.formatNmea = function( elem , record , column , data)
{
	switch( data )
	{
		case '0':
		{
			elem.innerHTML = '';
			break;		
		}
		case '1':
		{
			elem.innerHTML = getResource( 'NtripSourceTable-Resource-Requested' );
			break;			
		}
	}
};

NtripSourceTableView.formatSolution = function( elem , record , column , data)
{
	switch( data )
	{
		case '0':
		{
			elem.innerHTML = getResource( 'NtripSourceTable-Resource-SingleBase' );
			break;
		}
		case '1':
		{
			elem.innerHTML = getResource( 'NtripSourceTable-Resource-Network' );
			break;
		}
	}
};

var theNtripSourceTableView = null;

function fillNtripSourceTable(source, loading)
{
	// Store current mount
	var currentmount = getFieldValue('RoverSetup-Field-Network-Ntrip-Mount');
		
	if (source.NtripSourceTable.length > 0)
	{
		// Clear all values
		clearAllFieldOptions('RoverSetup-Field-Network-Ntrip-Mount');

		// Fill with new values
		for (i in source.NtripSourceTable)
		{
			var mount = source.NtripSourceTable[i];
			
			addFieldOption('RoverSetup-Field-Network-Ntrip-Mount', mount.MountPoint, mount.MountPoint);
		}
		
		//Automatic Selection of Mount Point thanks to JSON file
		if (currentmount == "") {
			var select = document.getElementById('RoverSetup-Field-Network-ConnectionType');
			var selectedValue = select.options[select.selectedIndex].value;
			
			switch (selectedValue) {
			case 3:
			case "3":
				currentmount = configGlobal.Network.NtripPortE.Mount;
			break;
			case 5:
			case "5":
				currentmount = configGlobal.Network.NtripPortP.Mount;
			break;
			}
		}
		
		// Re-select current mount if exists
		setFieldValue('RoverSetup-Field-Network-Ntrip-Mount', currentmount);
	}
	else
	{
		// Loading or empty
		setFieldOption('RoverSetup-Field-Network-Ntrip-Mount', 
			getResource(loading ? 
				'NtripSourceTable-Resource-Loading' :
				'NtripSourceTable-Resource-PleaseLoad'),
			"");
	}
		
	// Create table view
    theNtripSourceTableView = new NtripSourceTableView(source, loading);
	
	// Table view selection changed
	theNtripSourceTableView.ntripSourceDataTable.subscribe("rowSelectEvent", onNtripSourceTableViewSelChange);
	
	// Try to select current mount point
	if (!theNtripSourceTableView.selectMountPoint(currentmount))
	{
        // Programmatically select the first row
        theNtripSourceTableView.ntripSourceDataTable.selectRow(0);
	}
}

function onLoadSourceTable()
{
	// Define the callbacks for the asyncRequest
	var callbacks = {
		
		success : function (o) {

			YAHOO.log("RAW JSON DATA: " + o.responseText);

			// Process the JSON data returned from the server
			var data;
			try {
				data = YAHOO.lang.JSON.parse(o.responseText);
			}
			catch (x) {
				YAHOO.log("ERROR: JSON Parse failed!");

				// Retry
				//setTimeout("onLoadSourceTable()", 5000);

				return;
			}

			YAHOO.log("PARSED DATA: " + YAHOO.lang.dump(data));

			// Get source table
			var source = data.StatusNtripSourceTable;
			if (source == null) return;
			
			// Fill source table widgets
			fillNtripSourceTable(source, false);
		},

		failure : function (o) {
			if (!YAHOO.util.Connect.isCallInProgress(o)) {
				YAHOO.log("ERROR: Async call failed!");
			}
			
			// Fill source table widgets (no data)
			fillNtripSourceTable({NtripSourceTable:[]}, false);
		},

		timeout : 60000,
		cache : false
    };

	// Loading...
	fillNtripSourceTable({NtripSourceTable:[]}, true);
	
	// Make the call to the server for JSON data	
	var url = getStatusNtripSourceTableURL;
	if (simul != 1)
	{
		url += 
			"&ConnectionType=" + getFieldValue('RoverSetup-Field-Network-ConnectionType') +
			"&IP=" + getFieldValue('RoverSetup-Field-Network-Ntrip-AddressIP') + 
			"&Port=" + getFieldValue('RoverSetup-Field-Network-Ntrip-AddressPort') + 
			"&ReceiverModel=" + getFieldValue('RoverSetup-Field-Network-ReceiverModel');
	}
	
	YAHOO.util.Connect.asyncRequest('GET', url, callbacks);
}

function onLoadFormerSourceTable()
{
	var sourceTable = {NtripSourceTable:[]};
	fillNtripSourceTable(sourceTable, false);
	
	var urlSourceTableJSON = "/data/StatusNtripSourceTable.json";
	
	var callbacksSourceTableJSON = {
		success : function (o) {
			YAHOO.log("RAW JSON DATA: " + o.responseText);

			// Process the JSON data returned from the server
			var data;
			try {
				data = YAHOO.lang.JSON.parse(o.responseText);
			}
			catch (x) {
				YAHOO.log("ERROR: JSON Parse failed!");
				
				return;
			}

			// Get source table
			if (data.StatusNtripSourceTable == null)
				return;
			else
				sourceTable = data.StatusNtripSourceTable;
			
			var now = new Date();			
			var datetime = YAHOO.lang.JSON.stringToDate(sourceTable.Datetime);
			
			if (sourceTable.AddressIp == getFieldValue('RoverSetup-Field-Network-Ntrip-AddressIP') &&
				sourceTable.AddressPort == getFieldValue('RoverSetup-Field-Network-Ntrip-AddressPort') &&
				(Math.abs((now.getTime() - datetime.getTime()) < 86400000)))
			{
				fillNtripSourceTable(sourceTable, false);
			}
		},
		failure : function (o) {
			if (!YAHOO.util.Connect.isCallInProgress(o)) {
				YAHOO.log("ERROR: Async call failed!");
			}
		},
		timeout : 10000,
		cache : false
    };
	
	YAHOO.util.Connect.asyncRequest('GET', urlSourceTableJSON, callbacksSourceTableJSON);
}

function onDistanceUnitChanged()
{
	updateFieldValueAsDistance('RoverSetup-Field-AntennaHeight');
	updateFieldValueAsDistance('RoverSetup-Field-AntennaRadius');
	updateFieldValueAsDistance('RoverSetup-Field-AntennaSHMP');
}

function onModeChanged(ev)
{
    var field = YAHOO.util.Event.getTarget(ev);
    setFieldVisible('RoverSetup-Field-MovingMode', (field.value != 2));
    setItemDisplayed('RoverSetup-Group-HotStandbyRTK', (field.value < 2));
}

function onFastRtkClicked(ev)
{
	enableMovingBase(isFieldChecked('RoverSetup-Field-MovingMode'));
}

function onMovingModeClicked(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	enableMovingBase(isFieldChecked(field));
}

function onMeasurementTypeChanged(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	enableSHMP(field.value);
}

function onMeasurementTypeChanged2(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	enableSHMP2(field.value);
}

function onPortAConnectionTypeChanged(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	enablePortAParameters(field.value);
}

function onPortAModeChanged(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	enablePortARtsCts(field.value);
}

function onPortBConnectionTypeChanged(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);

	enablePortBParameters(field.value);
}

function onPortFConnectionTypeChanged(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);

	enablePortFParameters(field.value);
}

function onNetworkConnectionTypeChanged(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	enableNetworkParameters(field.value);
}

function enableMovingBase(enable)
{
	if (enable) {
		setFieldChecked('RoverSetup-Field-FastRTK', false); // Fast RTK impossible when Moving Base
	}
}

function enableSHMP(MeasurementType)
{
	setFieldVisible('RoverSetup-Group-SHMP',
		MeasurementType == 0 /* Slant Height */);	
}

function enableSHMP2(MeasurementType)
{
	setFieldVisible('RoverSetup-Group-SHMP2',
		MeasurementType == 0 /* Slant Height */);	
}

function enablePortDParameters(connection)
{
	setFieldVisible('RoverSetup-Field-Radio-Power-On', connection > 0);
	setFieldVisible('RoverSetup-Field-Radio-Power-Off', connection > 0);
}

function enablePortAParameters(connection)
{
	if ((connection == 5)||(connection == 6)) { // Licence Free
	    setFieldValue('RoverSetup-Field-PortA-BaudRate',5); // 9600 
	    enableField('RoverSetup-Field-PortA-BaudRate',false);
		
	    setFieldValue('RoverSetup-Field-PortA-Mode',232); // 232
	    enableField('RoverSetup-Field-PortA-Mode',false);

	} else {
	    enableField('RoverSetup-Field-PortA-BaudRate',true);
	    enableField('RoverSetup-Field-PortA-Mode',true);
	    
	    if (connection == 9)
	    {   // XDL Rover
	        setFieldValue('RoverSetup-Field-PortA-BaudRate',7); // 38400 
    	    setFieldValue('RoverSetup-Field-PortA-Mode',232); 
	        setFieldChecked('RoverSetup-Field-PortA-RtsCts', 0);
	    }
	}
	
	enablePortARtsCts(getFieldValue('RoverSetup-Field-PortA-Mode'));
}

function enablePortARtsCts(mode)
{
	setFieldVisible('RoverSetup-Field-PortA-RtsCts', mode == 232);
}

function enablePortBParameters(connection)
{
    if ((connection == 5)||(connection == 6)) { // Licence Free
	    setFieldValue('RoverSetup-Field-PortB-BaudRate',5); // 9600 
	    enableField('RoverSetup-Field-PortB-BaudRate',false);
	} else {
	    enableField('RoverSetup-Field-PortB-BaudRate',true);

	    if (connection == 9)
	    {   // XDL Rover
	        setFieldValue('RoverSetup-Field-PortB-BaudRate',7); // 38400 
	        setFieldChecked('RoverSetup-Field-PortB-RtsCts', 0);
	    }
	}
}

function enablePortFParameters(connection)
{
    if ((connection == 5)||(connection == 6)) { // Licence Free
	    setFieldValue('RoverSetup-Field-PortF-BaudRate',5); // 9600 
	    enableField('RoverSetup-Field-PortF-BaudRate',false);

	} else {
	    enableField('RoverSetup-Field-PortF-BaudRate',true);

	    if (connection == 9)
	    {   // XDL Rover
	        setFieldValue('RoverSetup-Field-PortF-BaudRate',7); // 38400 
	        setFieldChecked('RoverSetup-Field-PortF-RtsCts', 0);
	    }
	}
}

function enableNetworkParameters(connection)
{
	switch (connection) {
	case 2:
	case "2":
		// -- Network / Direct IP Port E --
		setFieldValue('RoverSetup-Field-Network-DirectIP-AddressIP', configGlobal.Network.DirectIPPortE.AddressIp);
		setFieldValue('RoverSetup-Field-Network-DirectIP-AddressPort', configGlobal.Network.DirectIPPortE.AddressPort);
		setFieldValue('RoverSetup-Field-Network-DirectIP-Login', configGlobal.Network.DirectIPPortE.Login);
		setFieldValue('RoverSetup-Field-Network-DirectIP-Password', configGlobal.Network.DirectIPPortE.Password);
	break;
	case 3:
	case "3":
		// -- Network / Ntrip Port E --
		setFieldValue('RoverSetup-Field-Network-Ntrip-AddressIP', configGlobal.Network.NtripPortE.AddressIp);
		setFieldValue('RoverSetup-Field-Network-Ntrip-AddressPort', configGlobal.Network.NtripPortE.AddressPort);
		setFieldOption('RoverSetup-Field-Network-Ntrip-Mount', configGlobal.Network.NtripPortE.Mount, configGlobal.Network.NtripPortE.Mount);
		setFieldValue('RoverSetup-Field-Network-Ntrip-Mount', configGlobal.Network.NtripPortE.Mount);
		setFieldValue('RoverSetup-Field-Network-Ntrip-Login', configGlobal.Network.NtripPortE.Login);
		setFieldValue('RoverSetup-Field-Network-Ntrip-Password', configGlobal.Network.NtripPortE.Password);
		setFieldChecked('RoverSetup-Field-Network-Ntrip-SendNMEA', configGlobal.Network.NtripPortE.SendNmea);
		
		onLoadFormerSourceTable();
	break;
	case 4:
	case "4":
		// -- Network / Direct IP Port P --
		setFieldValue('RoverSetup-Field-Network-DirectIP-AddressIP', configGlobal.Network.DirectIPPortP.AddressIp);
		setFieldValue('RoverSetup-Field-Network-DirectIP-AddressPort', configGlobal.Network.DirectIPPortP.AddressPort);
		setFieldValue('RoverSetup-Field-Network-DirectIP-Login', configGlobal.Network.DirectIPPortP.Login);
		setFieldValue('RoverSetup-Field-Network-DirectIP-Password', configGlobal.Network.DirectIPPortP.Password);
	break;
	case 5:
	case "5":
		// -- Network / Ntrip Port P --
		setFieldValue('RoverSetup-Field-Network-Ntrip-AddressIP', configGlobal.Network.NtripPortP.AddressIp);
		setFieldValue('RoverSetup-Field-Network-Ntrip-AddressPort', configGlobal.Network.NtripPortP.AddressPort);
		setFieldOption('RoverSetup-Field-Network-Ntrip-Mount', configGlobal.Network.NtripPortP.Mount, configGlobal.Network.NtripPortP.Mount);
		setFieldValue('RoverSetup-Field-Network-Ntrip-Mount', configGlobal.Network.NtripPortP.Mount);
		setFieldValue('RoverSetup-Field-Network-Ntrip-Login', configGlobal.Network.NtripPortP.Login);
		setFieldValue('RoverSetup-Field-Network-Ntrip-Password', configGlobal.Network.NtripPortP.Password);
		setFieldChecked('RoverSetup-Field-Network-Ntrip-SendNMEA', configGlobal.Network.NtripPortP.SendNmea);
		
		onLoadFormerSourceTable();
	break;	
	}
	
	setFieldVisible('RoverSetup-Field-Network-PhoneNumber',
		connection == 1 /* Modem CSD */);
		
	setItemDisplayed('RoverSetup-Group-Network-DirectIP',
		connection == 2 /* Modem Direct IP */ ||
		connection == 4 /* Ethernet Direct IP */);

	setItemDisplayed('RoverSetup-Group-Network-Ntrip',
		connection == 3 /* Modem Ntrip client */ ||
		connection == 5 /* Ethernet Ntrip client */);	
		
	//if (connection == 3) { /* Modem Ntrip client */
	//    setFieldValue('RoverSetup-Field-Network-Ntrip-Login',"");
	//}
	
	setFieldVisible('RoverSetup-Field-Network-ConnectNow', connection > 0);		
}

function onNtripHostChange(ev)
{
	// Clear combo
	setFieldOption('RoverSetup-Field-Network-Ntrip-Mount', getResource('NtripSourceTable-Resource-PleaseLoad'), "");
	
	// Destroy table view if any
	if (theNtripSourceTableView != null)
	{
		theNtripSourceTableView.destroy();
	}
	
	YAHOO.util.Cookie.remove("sourceTableSize");
	YAHOO.util.Cookie.remove("sourceTableConnectionType");
	YAHOO.util.Cookie.remove("sourceTableIP");
	YAHOO.util.Cookie.remove("sourceTablePort");
}

function onNtripMountChange(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	if (theNtripSourceTableView != null)
	{
		theNtripSourceTableView.selectMountPoint(field.value);
	}
}

function onNtripSourceTableViewSelChange()
{
	// Get mount point
	var mount = theNtripSourceTableView.getSelectedMountPoint();
	
	// Select mount point
	setFieldValue('RoverSetup-Field-Network-Ntrip-Mount', mount != null ? mount.MountPoint : "");
	
	// Check if Send NMEA is needed
	setFieldChecked('RoverSetup-Field-Network-Ntrip-SendNMEA', mount != null && mount.NMEA == "1");
}

function onDifferentialPortModeChanged(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	
	// Get mode
	var mode = getRadioButtonCheckedValue(
		new Array(
			'RoverSetup-Field-DifferentialPortMode-Auto', 
			'RoverSetup-Field-DifferentialPortMode-Manual'));
					
	enableDifferentialStreams(mode);
}

function enableDifferentialStreams(mode)
{
	// Enable when mode = Manual
	setFieldVisible('RoverSetup-Field-Stream1-Port', mode == 0);
	setFieldVisible('RoverSetup-Field-Stream2-Port', mode == 0);
}

function onHotStandbyRTKClicked(ev)
{
	var field = YAHOO.util.Event.getTarget(ev);
	var checked = (field.checked == true) ? 1 : 0;
					
	enableHotStandbyRTKParameters(checked);
}

function enableHotStandbyRTKParameters(checked)
{
	setFieldVisible('RoverSetup-Field-HotStandbyRTK-Stream-Port', checked == 1);
}

function onFieldGPSChange( ev )
{
	var field = YAHOO.util.Event.getTarget(ev);
	var field_GLO = YAHOO.util.Dom.get(getPrefixName() + '-Field-GLONASS');
	
	if ((isFieldChecked(field) == 0) && (isFieldChecked(field_GLO) == 0)) 
	{
	    setFieldChecked(getPrefixName() + '-Field-GPS', 1);
	}
}

function onFieldGLONASSChange( ev )
{
	var field = YAHOO.util.Event.getTarget(ev);
	var field_GPS = YAHOO.util.Dom.get(getPrefixName() + '-Field-GPS');
	
	if (!isFieldChecked(field) && !isFieldChecked(field_GPS)) 
	{
	    setFieldChecked(getPrefixName() + '-Field-GLONASS', 1);
	}
}