function updateStatusAlarms()
{
	// Define the callbacks for the asyncRequest
	var callbacks = {

		revive : function (k,v) {

			if (k == "GmtDateTime")
			{
				try
				{
					return YAHOO.lang.JSON.stringToDate(v);
				}
				catch (x) {
					YAHOO.log("ERROR: JSON revive failed!");
				}
			}
	
			return v;
		},

		success : function (o) {

			YAHOO.log("RAW JSON DATA: " + o.responseText);

			// Process the JSON data returned from the server
			var data;
			try {
				data = YAHOO.lang.JSON.parse(o.responseText, callbacks.revive);
			}
			catch (x) {
				YAHOO.log("ERROR: JSON Parse failed!");

				// Retry to update
				setTimeout("updateStatusAlarms()", 5000);

				return;
			}

			YAHOO.log("PARSED DATA: " + YAHOO.lang.dump(data));			

			// Fill table
			var theAlarmTableView = new AlarmTableView(data.StatusAlarms.Alarms);

			// Update every 5 seconds
			setTimeout("updateStatusAlarms()", 5000);
		},

		failure : function (o) {
			if (!YAHOO.util.Connect.isCallInProgress(o)) {
				YAHOO.log("ERROR: Async call failed!");
			}
			
			// Retry to update
			setTimeout("updateStatusAlarms()", 5000);
		},

		timeout : 10000,
		cache : false
	};

	// Make the call to the server for JSON data
	YAHOO.util.Connect.asyncRequest('GET', getStatusAlarms, callbacks);
}

function AlarmTableView(source)
{
	// Columns definition
	var myColumnDefs = [
		{label:getResource('Alarms-Resource-GmtDateTime'), key:"GmtDateTime", formatter:AlarmTableView.formatGmtDateTime, resizeable:true},
		{label:getResource('Alarms-Resource-Code'),    key:"Code", formatter:YAHOO.widget.DataTable.formatNumber, resizeable:true},
		{label:getResource('Alarms-Resource-SubCode'), key:"SubCode", formatter:YAHOO.widget.DataTable.formatNumber, resizeable:true},
		{label:getResource('Alarms-Resource-Message'), key:"Code", formatter:AlarmTableView.formatMessageLabel, resizeable:true}
	];

	// Data source
	var myDataSource = new YAHOO.util.DataSource(source);
	myDataSource.responseType = YAHOO.util.DataSource.TYPE_JSARRAY;
	myDataSource.responseSchema = {
		fields: ["GmtDateTime","Code","SubCode"]
	};

	// Table configs
	var myConfigs = {
		width:"600px"
	};
	
	// Localized messages
	myConfigs.MSG_EMPTY = getResource('Alarms-Resource-Empty');
	myConfigs.MSG_LOADING = getResource('Alarms-Resource-Loading');

	// Create table
	var myDataTable = new YAHOO.widget.DataTable("AlarmTableView", myColumnDefs, myDataSource, myConfigs);
	
	function updateTable()
	{
		myDataTable.showTableMessage("Loading...");
		myDataTable.getDataSource().sendRequest('', { success: myDataTable.onDataReturnInitializeTable, scope: myDataTable });
	}
	
	YAHOO.util.Event.addListener('timezone', 'change', updateTable );
}

AlarmTableView.formatGmtDateTime = function(el, oRecord, oColumn, oData)
{
	try
	{
		
		el.innerHTML = datetimeToCurrentTimezone( oData ).full;
	}
	catch (x) {
		YAHOO.log("AlarmTableView.formatGmtDateTime: '" + oData + "' bad format");
	}
};

AlarmTableView.formatMessageLabel = function(el , oRecord , oColumn , oData)
{
	el.innerHTML = getResource('Alarms-Resource-Code-' + oData);
};