var theEmbeddedCasterSourceView = null;
var theEmbeddedCasterClientView = null;
var myTabs = null;

var currentStatus = null;

function getSourcesStatus()
{
	if (currentStatus != null)
	{
		for (i in currentStatus.Sources)
		{
			//Status Icon
			var iconClass = null;
			switch (currentStatus.Sources[i].Status)
			{
			case 0:
				iconClass = "disconnect";
				break;
			case 1:
				iconClass = "intruder";
				break;
			case 2:
				iconClass = "connect";			
				break;			
			}
			if (iconClass != null)
				currentStatus.Sources[i].Status = '<span class="' + iconClass + '">&nbsp;</span>';
			
			//Time format
			var date = currentStatus.Sources[i].StartTime;
			
			if (date != null) {
				if (date.indexOf("UTC") != -1) {
					date = datetimeToCurrentTimezone(date.replace(" UTC", "Z").replace(" ", "T"));
					date = date.date + " " + date.time;
					currentStatus.Sources[i].StartTime = date;
				} else {
					break;
				}
			} else {
				currentStatus.Sources[i].StartTime = "";
			}
				
		}
		
		return currentStatus.Sources;
	}
	else return [];
}

function getClientsStatus()
{
	if (currentStatus != null)
	{
		for (i in currentStatus.Clients)
		{
			var date = currentStatus.Clients[i].StartTime;
			
			if (date.indexOf("UTC") != -1) {
				date = datetimeToCurrentTimezone(date.replace(" UTC", "Z").replace(" ", "T"));
				date = date.date + " " + date.time;
				currentStatus.Clients[i].StartTime = date;
			} else {
				break;
			}
		}
		
		return currentStatus.Clients;
	}
	else return [];
}

function updateStatusEmbeddedCasterCurrent()
{
	// Define the callbacks for the asyncRequest
	var callbacks = {
		success : function (o) {
			
			// Process the JSON data returned from the server
			var data;
			try {
				data = YAHOO.lang.JSON.parse(o.responseText);
			}
			catch (x) {
				YAHOO.log("ERROR: JSON Parse failed!");

				// Retry
				setTimeout("updateStatusEmbeddedCasterCurrent()", 5000);

				return;
			}
			
			// Get config
			var status = data.StatusEmbeddedCasterCurrent;
			if (status == null) return;
			
			currentStatus = status;
			
			//Tabs
			if (myTabs == null)
			{
				myTabs = new YAHOO.widget.TabView("tabs");
			}
			
			//Fill Table
			if (theEmbeddedCasterSourceView == null)
				theEmbeddedCasterSourceView = new EmbeddedCasterSourceView(getSourcesStatus);
			
			if (theEmbeddedCasterClientView == null)
				theEmbeddedCasterClientView = new EmbeddedCasterClientView(getClientsStatus);

			// Update every 5 seconds
			setTimeout("updateStatusEmbeddedCasterCurrent()", 5000);
		},

		failure : function (o) {
			if (!YAHOO.util.Connect.isCallInProgress(o)) {
				YAHOO.log("ERROR: Async call failed!");
			}
		},

		timeout : 10000,
		cache : false
	};
	
	// Make the call to the server for JSON data
	YAHOO.util.Connect.asyncRequest('GET', getStatusEmbeddedCasterCurrent, callbacks);
}

function EmbeddedCasterSourceView(source)
{
	// Columns definition
	var myColumnDefs = [
		{label:getResource('EmbeddedCaster-Source-Resource-Source'), key:"MountPoint", resizeable:true},
		{label:getResource('EmbeddedCaster-Source-Resource-Status'), key:"Status", resizeable:true, className:"centered"},
		{label:getResource('EmbeddedCaster-Source-Resource-StartTime'), key:"StartTime", resizeable:true},
		{label:getResource('EmbeddedCaster-Source-Resource-IP'), key:"AddressIP", resizeable:true}
	];

	// Data source
	var myDataSource = new YAHOO.util.DataSource(source);
	myDataSource.responseType = YAHOO.util.DataSource.TYPE_JSARRAY;
	myDataSource.responseSchema = {
		fields: ["MountPoint","Status","StartTime","AddressIP"]
	};

	// Table configs
	var myConfigs = {
		width:"auto"
	};
	
	// Localized messages
	myConfigs.MSG_EMPTY = getResource('EmbeddedCaster-Source-Resource-Empty');
	myConfigs.MSG_LOADING = getResource('EmbeddedCaster-Source-Resource-Loading');

	// Create table
	var myDataTable = new YAHOO.widget.DataTable("EmbeddedCaster-Source-Table", myColumnDefs, myDataSource, myConfigs);
	
	// Set up polling
	var myCallback =
	{
		success: myDataTable.onDataReturnInitializeTable,
		failure: function()
		{
			YAHOO.log("EmbeddedCasterSourceView: Polling failure", "error");
		},
		scope: myDataTable
	};
	myDataSource.setInterval(5000, null, myCallback);
}

function EmbeddedCasterClientView(source)
{
	// Columns definition
	var myColumnDefs = [
		{label:getResource('EmbeddedCaster-Client-Resource-User'), key:"User", resizeable:true},
		{label:getResource('EmbeddedCaster-Client-Resource-Source'), key:"MountPoint", resizeable:true},
		{label:getResource('EmbeddedCaster-Client-Resource-StartTime'), key:"StartTime", resizeable:true},
		{label:getResource('EmbeddedCaster-Client-Resource-IP'), key:"AddressIP", resizeable:true}
	];

	// Data source
	var myDataSource = new YAHOO.util.DataSource(source);
	myDataSource.responseType = YAHOO.util.DataSource.TYPE_JSARRAY;
	myDataSource.responseSchema = {
		fields: ["User","MountPoint","StartTime","AddressIP"]
	};

	// Table configs
	var myConfigs = {
		width:"auto"
	};
	
	// Localized messages
	myConfigs.MSG_EMPTY = getResource('EmbeddedCaster-Client-Resource-Empty');
	myConfigs.MSG_LOADING = getResource('EmbeddedCaster-Source-Resource-Loading');

	// Create table
	var myDataTable = new YAHOO.widget.DataTable("EmbeddedCaster-Client-Table", myColumnDefs, myDataSource, myConfigs);

	// Set up polling
	var myCallback =
	{
		success: myDataTable.onDataReturnInitializeTable,
		failure: function()
		{
			YAHOO.log("EmbeddedCasterClientView: Polling failure", "error");
		},
		scope: myDataTable
	};
	myDataSource.setInterval(5000, null, myCallback);
}