var map;
var controls;
var OSMLayer, sourceLayer, clientLayer, baselineLayer;
var initCenter = 0;
var projectionWGS84 = new OpenLayers.Projection("EPSG:4326");

var baseFeature, sourceFeature, clientFeature, baselineFeature;

var baselineStyle;

//Enum
var typeBase = 0, typeSource = 1, typeClient = 2;

baselineStyle = {
    strokeColor: "#017dc3",
    strokeWidth: 3,
    strokeDashstyle: "solid",
    pointRadius: 3,
    pointerEvents: "visiblePainted"
};

// Prototype
OpenLayers.Feature.Vector.prototype.type = null;
OpenLayers.Feature.Vector.prototype.cid = null;
OpenLayers.Feature.Vector.prototype.login = null;
OpenLayers.Feature.Vector.prototype.ip = null;
OpenLayers.Feature.Vector.prototype.lonlat = null;
OpenLayers.Feature.Vector.prototype.source = null;
OpenLayers.Feature.Vector.prototype.name = null;
OpenLayers.Feature.Vector.prototype.identifier = null;
OpenLayers.Feature.Vector.prototype.format = null;
OpenLayers.Feature.Vector.prototype.height = null;
OpenLayers.Feature.Vector.prototype.positiontype = null;
OpenLayers.Feature.Vector.prototype.internals = null;

function getCurrentOpenLayersLonLat()
{
	var currentPosition = getCurrentPosition();
    return new OpenLayers.LonLat(currentPosition[1], currentPosition[0]).transform(projectionWGS84,map.getProjectionObject());
}

function createPopup(feature) {
	var description = "";
	var lonlat84 = feature.lonlat.clone().transform(map.getProjectionObject(),projectionWGS84);
	var formattedLat = formatTextAsAngle(lonlat84.lat, 'S', 'N');
	var formattedLon = formatTextAsAngle(lonlat84.lon, 'W', 'E');
	switch (feature.type)
	{
	//Base
	case typeBase:
		var mountpoints = "";
		for (j in feature.internals)
		{
			if (mountpoints != "")
				mountpoints += ", ";
			mountpoints += feature.internals[j].Name;
		}
		
		description = "<span class=\"EmbeddedCasterMapPopupContentBase\">Base</span><br/>"
					+ "Sources: " + mountpoints + "<br />"
					+ "Lat: " + formattedLat + "<br />"
					+ "Lon: " + formattedLon;
	break;
	//MountPoint
	case typeSource:
		description = "<span class=\"EmbeddedCasterMapPopupContentSource\">Source</span><br/>"
					+ "Name: " + feature.name + "<br />"
					+ "Identifier: " + feature.identifier + "<br />"
					+ "Format: " + feature.format + "<br />"
					+ "Lat: " + formattedLat + "<br />"
					+ "Lon: " + formattedLon;
	break;
	//Client	
	case typeClient:
		var position = "";
		switch (parseInt(feature.positiontype,10))
		{
		case 1:
			position = "Autonomous";
		break;
		case 2:
			position = "DGPS";
			break;
		case 3:
			position = "S-DGPS";
			break;
		case 4:
			position = "RTK Fixed";
			break;
		case 5:
			position = "RTK Float";
			break;
		case 9:
			position = "RTK Flying";
			break;
		case 0:
		default:
			position = "No position";
		break;
		}
		description = "<span class=\"EmbeddedCasterMapPopupContentClient\">Client</span><br/>"
					+ "Login: " + feature.login + "<br />" 
					+ "IP: " + feature.ip + "<br />"
					+ "Source: " + feature.source + "<br />"
					+ "Lat: " + formattedLat + "<br />"
					+ "Lon: " + formattedLon + "<br />"
					+ "Alt: " + formatTextAsDistance(feature.height) + "<br />"
					+ "Status: " + position;
	break;
	}
	feature.popup = new OpenLayers.Popup.FramedCloud("pop",
		feature.geometry.getBounds().getCenterLonLat(),
		null,
		'<div class="EmbeddedCasterMapPopupContent">'+description+'</div>',
		null,
		true,
		function() { controls['selector'].unselectAll(); }
	);
	map.addPopup(feature.popup);
}

function destroyPopup(feature) {
	feature.popup.destroy();
	feature.popup = null;
}

function formatLonlat(lonLat) {
    return formatTextAsAngle(lonLat.lat, 'S', 'N') + ", " + 
    		formatTextAsAngle(lonLat.lon, 'W', 'E');
}

function initStatusEmbeddedCasterMap()
{
	// Clean <div>
	document.getElementById("EmbeddedCasterMap").innerHTML = '';
	
	//Vérifier le bon fonctionnement de navigator.onLine
	//Si ce n'est pas supporté, tester un xhr
	map = new OpenLayers.Map("EmbeddedCasterMap", { 
		controls: [new OpenLayers.Control.Navigation(),
		           new OpenLayers.Control.PanZoomBar(),
		           new OpenLayers.Control.LayerSwitcher(),
		           new OpenLayers.Control.MousePosition({formatOutput: formatLonlat}),
		           new OpenLayers.Control.OverviewMap(),
		           new OpenLayers.Control.KeyboardDefaults()],
		displayProjection: projectionWGS84
	});
	
	OSMLayer = new OpenLayers.Layer.OSM();
    sourceLayer = new OpenLayers.Layer.Vector("Sources");
    clientLayer = new OpenLayers.Layer.Vector("Clients");
    baselineLayer = new OpenLayers.Layer.Vector("Baselines");

    map.addLayers([OSMLayer, sourceLayer, clientLayer, baselineLayer]);
	
    var statusBarLonLat = getCurrentOpenLayersLonLat();
    map.setCenter(statusBarLonLat, 10);
    
    baseFeature = new OpenLayers.Feature.Vector(new OpenLayers.Geometry.Point(0, 0),
    	{},{externalGraphic: '../../images/orange.gif', graphicHeight: 16, graphicWidth: 16, graphicXOffset:-8, graphicYOffset:-8 }
    );
    baseFeature.move(statusBarLonLat);
    //Prototype
    baseFeature.lonlat = statusBarLonLat;
    baseFeature.type = typeBase;
    sourceLayer.addFeatures(baseFeature);

    //Add selector controls to the vectorLayers with popup functions
    controls = {
      selector: new OpenLayers.Control.SelectFeature([sourceLayer,clientLayer], { onSelect: createPopup, onUnselect: destroyPopup })
    };

    map.addControl(controls['selector']);
    controls['selector'].activate();
    
	setTimeout("updateStatusEmbeddedCasterMap()", 2000);
}

var markerMouseDown = function(evt) {
	this.popup.lonlat = this.marker.lonlat.clone();
	var html = (this.marker.lonlat.clone().transform(map.getProjectionObject(),projectionWGS84) + "").replace(",","\n");
	this.popup.setContentHTML(html);
	this.popup.toggle();
	
    OpenLayers.Event.stop(evt);
};

// Add/Update Source
function processCasterSource(source) 
{
    if (source.Internal == false) 
    {
		var lonlat = new OpenLayers.LonLat(source.Longitude, source.Latitude).transform(projectionWGS84,map.getProjectionObject());	
		var foundFeature = sourceLayer.getFeatureBy('name',source.Name);

		var here_img = "../../images/go.gif";
		if (source.Status != 2) here_img = "../../images/stop.gif";

		if (foundFeature == null)
		{
			// New feature => Add
		    var here_img = "../../images/go.gif";
		    if (source.Status != 2) here_img = "../../images/stop.gif";
		    sourceFeature = new OpenLayers.Feature.Vector(new OpenLayers.Geometry.Point(0, 0),
		        {}, { externalGraphic: here_img, graphicHeight: 16, graphicWidth: 16, graphicXOffset: -8, graphicYOffset: -8 }
		    );
			
			// Prototype
			sourceFeature.type = typeSource;
			sourceFeature.name = source.Name;
			sourceFeature.identifier = source.Identifier;
			sourceFeature.format = source.Format;
			sourceFeature.lonlat = lonlat;
		    sourceLayer.addFeatures(sourceFeature);	    
			sourceFeature.move(sourceFeature.lonlat);
		}
		else
		{
			// Existing feature => Update
			foundFeature.identifier = source.Identifier;
			foundFeature.format = source.Format;
			foundFeature.lonlat = lonlat;
			foundFeature.style.externalGraphic = here_img;
			foundFeature.move(foundFeature.lonlat);
		}
	}
}

// Add/Update Client
function processCasterClient(client, sources)
{	
	var lonlat = new OpenLayers.LonLat(client.Longitude, client.Latitude).transform(projectionWGS84,map.getProjectionObject());
	
	// Build Baseline Feature
	var sourceLonlat = null;
	for (k in sources)
	{
		if (sources[k].Name == client.Source.substring(1,client.Source.length))
		{
			if (sources[k].Internal == true)
			{
				sourceLonlat = getCurrentOpenLayersLonLat();
			}
			else
			{
				sourceLonlat = new OpenLayers.LonLat(sources[k].Longitude, sources[k].Latitude).transform(projectionWGS84,map.getProjectionObject());
			}
			break;
		}
	}
	
	var pointList = [];
	var newPoint;
	// Client
    newPoint = new OpenLayers.Geometry.Point(lonlat.lon,lonlat.lat);
    pointList.push(newPoint);
    // Source
    newPoint = new OpenLayers.Geometry.Point(sourceLonlat.lon,sourceLonlat.lat);
    pointList.push(newPoint);
	
	var foundClientFeature = clientLayer.getFeatureBy('cid',client.Id);
	if (foundClientFeature == null)
	{
		// New feature => Add Client
		clientFeature = new OpenLayers.Feature.Vector(new OpenLayers.Geometry.Point(0, 0),
	        {},{externalGraphic: '../../images/stop.gif', graphicHeight: 16, graphicWidth: 16, graphicXOffset:-8, graphicYOffset:-8 }
	    );
		
		// Prototype
		clientFeature.type = typeClient;
		clientFeature.cid = client.Id;
		clientFeature.login = client.Login;
		clientFeature.ip = client.Ip;
		clientFeature.height = parseFloat(client.Altitude,10) + parseFloat(client.GeoidSeparation,10);
		clientFeature.positiontype = client.PositionType;
		clientFeature.lonlat = lonlat;
		clientFeature.source = client.Source.substring(1,client.Source.length);
	    clientLayer.addFeatures(clientFeature);
	    
		clientFeature.move(clientFeature.lonlat);
		
		// New feature => Add Baseline
	    baselineFeature = new OpenLayers.Feature.Vector(new OpenLayers.Geometry.LineString(pointList),null,baselineStyle);
	    baselineFeature.cid = client.Id;
	    baselineLayer.addFeatures(baselineFeature);
	    
	    /*
	    // New feature => Add Baseline Arrows
	    // Get Angle
	    var TWOPI = Math.PI * 2;
	    var dx = lonlat.lon - sourceLonlat.lon;
	    var dy = lonlat.lat - sourceLonlat.lat;
	    
	    var rad = Math.atan2(dy, dx);	    
	    if (rad < 0) rad += TWOPI;
	    // Reverse
	    rad = (rad + Math.PI) % (Math.PI * 2);
	    // Arrow
	    // 15% du segment
	    var arrowLength = dx / 100 * 20;
	    var arrowAngle = (Math.PI / 4) % TWOPI;
	    
	    // First Line of the Arrow
	    var firstAngle = rad + arrowAngle;
	    firstAngle %= TWOPI;
	    var firstArrowX = lonlat.lon + arrowLength * Math.cos(firstAngle);
	    var firstArrowY = lonlat.lat + arrowLength * Math.sin(firstAngle);

	    pointList = [];
	    newPoint = new OpenLayers.Geometry.Point(lonlat.lon, lonlat.lat);
	    pointList.push(newPoint);
	    newPoint = new OpenLayers.Geometry.Point(firstArrowX, firstArrowY);
	    pointList.push(newPoint);
	    baselineFeature = new OpenLayers.Feature.Vector(new OpenLayers.Geometry.LineString(pointList),null,baselineStyle);
	    baselineFeature.cid = client.Id + "_a";
	    baselineLayer.addFeatures(baselineFeature);
	    
	    // Second Line of the Arrow
	    var secondAngle = rad - arrowAngle;
	    if (secondAngle < 0) secondAngle += TWOPI;
	    var secondArrowX = lonlat.lon + arrowLength * Math.cos(secondAngle);
	    var secondArrowY = lonlat.lat + arrowLength * Math.sin(secondAngle);

	    pointList = [];
	    newPoint = new OpenLayers.Geometry.Point(lonlat.lon, lonlat.lat);
	    pointList.push(newPoint);
	    newPoint = new OpenLayers.Geometry.Point(secondArrowX, secondArrowY);
	    pointList.push(newPoint);
	    baselineFeature = new OpenLayers.Feature.Vector(new OpenLayers.Geometry.LineString(pointList),null,baselineStyle);
	    baselineFeature.cid = client.Id + "_b";
	    baselineLayer.addFeatures(baselineFeature);
	    */
	}
	else
	{
		// Existing feature => Update Client
		foundClientFeature.lonlat = lonlat;
		foundClientFeature.move(foundClientFeature.lonlat);
		
		// Existing feature => Update Baseline
		var foundBaselineFeature = baselineLayer.getFeatureBy('cid',client.Id);
		var formerPointList = foundBaselineFeature.geometry.getVertices();
		
		//pointList
		formerPointList[0].move(lonlat.lon - formerPointList[0].x, lonlat.lat - formerPointList[0].y);
		formerPointList[1].move(sourceLonlat.lon - formerPointList[1].x, sourceLonlat.lat - formerPointList[1].y);
		foundBaselineFeature.layer.drawFeature(foundBaselineFeature);
	}
}

// Delete former Rovers

function cleanLayers(sources, clients)
{
	for (j in sourceLayer.features)
	{
		// Avoid Base
		if (j == 0)
		{
			continue;
		}
		var foundLine = false;
		for (var k in sources) {
			
			if ((sourceLayer.features[j].name == sources[k].Name) || (sources[k].Internal == true))
		    {
				foundLine = true;
				break;
			}
		}
		if (foundLine == false)
		{
			// Delete
			sourceLayer.removeFeatures(sourceLayer.features[j]);
		}
	}
	
	for (j in clientLayer.features)
	{
		var foundLine = false;
		for (var k in clients) {			
			if(clientLayer.features[j].cid == clients[k].Id)
		    {
				foundLine = true;
				break;
			}
		}
		if (foundLine == false)
		{
			// Delete
			clientLayer.removeFeatures(clientLayer.features[j]);
			baselineLayer.removeFeatures(baselineLayer.features[j]);
		}
	}
}

function updateStatusEmbeddedCasterMap()
{
	var callbacks = {
		success : function (o) {
			
		    var statusBarLonLat = getCurrentOpenLayersLonLat();
			
			if (initCenter < 1)
			{
			    map.setCenter(statusBarLonLat, map.getZoom());
				initCenter++;
			}
			
			baseFeature.lonlat = statusBarLonLat;
			baseFeature.move(baseFeature.lonlat);
			
			// Process the JSON data returned from the server
			var data;
			try
			{
				data = YAHOO.lang.JSON.parse(o.responseText);
			}
			catch (x)
			{
				// Retry
				setTimeout("updateStatusEmbeddedCasterMap()", 10000);

				return;
			}
			
			var sources = data.StatusEmbeddedCasterMap.MountPointList;
			var clients = data.StatusEmbeddedCasterMap.Clients;
			var internals = data.StatusEmbeddedCasterMap.InternalMountPointList;
			
			baseFeature.internals = internals;
			
			for (var j in sources) {
				sources[j].Internal = false;
				//Specify internal MountPoints: P & Q
				for (k in internals)
				{
					if (sources[j].Name == internals[k].Name)
					{
						sources[j].Internal = true;
					}
				}
				
				processCasterSource(sources[j]);						    
			}
			
			for (var j in clients) {
			    processCasterClient(clients[j], sources);						    
			}
			
			cleanLayers(sources, clients);
			
			// Update every x seconds
			setTimeout("updateStatusEmbeddedCasterMap()", 5000);
		},

		failure : function (o) {
			if (!YAHOO.util.Connect.isCallInProgress(o)) {
				YAHOO.log("ERROR: Async call failed!");
			}

			setTimeout("updateStatusEmbeddedCasterMap()", 10000);
		},

		timeout : 15000,
		cache : false
	};
	
	// Make the call to the server for JSON data
	YAHOO.util.Connect.asyncRequest('GET', getStatusEmbeddedCasterMap, callbacks);
}